#include "ui_mainwindow.h"
#include "mainwindow.h"
#include "lyricsexternal.h"

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), ui(new Ui::MainWindow) {

	ui->setupUi(this);

	QCoreApplication::setApplicationVersion("0.9.3");
	QCoreApplication::setApplicationName("RRipper");

	// The variable 'version' is global available
	version = "RRipper 0.9.3";
	this->setWindowTitle(version);
	this->setWindowIcon(QIcon(":/images/Logo_100.png"));

	lyricsSource = 0;
	musicsonglyricsTRY = 1;

	QApplication::setQuitOnLastWindowClosed(false);

	LoadGlobalPreferences();

	// Add the toolbar (because of submenus in the menu this isn't possible to do in designer)
	CreateToolbar();
	createTrayIcon();

	external = new Lyrics;

	this->showMaximized();
	windowMaximised = 1;
	quitOrCloseRRipper = 0;

	// The processes are reset and as well the states of the stations (recording,playing,etc.)
	stations_processes.clear();
	values.clear();

	// The statusses. This makes them everywhere the same with only one time the need to be tranlated
	status_notconnected = tr("Not Connected...");
	status_connecting = tr("Connecting...");
	status_receivingdetails = tr("Receiving Details...");
	status_gettingtrackname = tr("Receiving Trackname...");
	status_buffering = tr("Buffering...");
	status_skipping = tr("Skipping...");
	status_ripping = tr("Ripping...");
	status_playing = tr("Playing...");

	// The same with the Standard title/artist text
	id3_title = tr("Title");
	id3_artist = tr("Artist");

	// All the database errors have the same structure. This is the title (ERR) and the first part of the message (ERRT)
	ERR = tr("Error");
	ERRT = tr("An error occured!");

	// Some global variables:
	streamripperforplaying = new QProcess;
	mplayer = new QProcess;			// The mplayer process for the playback
	played = "";				// The code of the stream currently played
	torestart = -1;				// When a stream is played and the recording is started, here is saved, that the stream is automatically restarted
	IComeFromDisplayStations = 0;		// Prevents a loop of the functions DirectoriesItemExpanded() and DisplayStations()
	lock = new QTimer(this);		// Every 2 seconds a timestamp in a file is updated to show, that an instance of RRipper is running
	lock->start(1000);			// .... start this timer right NOW
	playingFileID3 = "";			// For the intelligent playback we need the filename of the file globally saved.
	CurrentCollapsingOutOfFunction = 0;	// The collapsing function (of course) emits as well the collapsed-SIGNAL
	volumeBeforeMuting = -1;

	// The basic layout of the station list
	ui->Directories_list->setColumnCount(2);
	ui->Directories_list->hideColumn(1);		// Here the id is saved, not visible for the normal user -> hidden
	codeOfLastAdded = "";				// When a new station is added here the new code is saved (important for adding a station of the directories list to the list of saved stations)

	// The columns in the station list is flexible adjusted to the available width
	ResizeColumns();

	// The corner left button removes the sorting of the list.
	QAbstractButton* cornerButton = ui->list->findChild<QAbstractButton*>();
	cornerButton->setToolTip(tr("Remove Sorting"));

	// Set up the column headers

	// 1) The symbol column
	QTableWidgetItem *header = new QTableWidgetItem("",0);
	ui->list->setHorizontalHeaderItem(0,header);

	// 2) The status column
	header = new QTableWidgetItem(tr("Status",0));
	ui->list->setHorizontalHeaderItem(1,header);

	// 3) The stream name column
	header = new QTableWidgetItem(tr("Stream",0));
	ui->list->setHorizontalHeaderItem(2,header);

	// 4) The artist column
	header = new QTableWidgetItem(tr("Artist",0));
	ui->list->setHorizontalHeaderItem(3,header);

	// 5) The title column
	header = new QTableWidgetItem(tr("Title",0));
	ui->list->setHorizontalHeaderItem(4,header);

	// 6) The file size column
	header = new QTableWidgetItem(QString::fromUtf8("File Size"),0);
	ui->list->setHorizontalHeaderItem(5,header);


	// These two streamripper-pref-fields have a regexp, a colored background shows if it's okay or not (white background when checkbox not activated)
	// Problem: If the fields don't have a background color from the beginning, an deactivating and again activating of the checkboxes (without changing the station selection) would remove the content, if they didn't have already a background color (maybe a bug of QT?)
	QPalette palette;
	palette.setColor(ui->Pref_D_Details->backgroundRole(),QColor(255,255,255));
	ui->Pref_D_Details->setPalette(palette);
	palette.setColor(ui->Pref_a_Details->backgroundRole(), QColor(255,255,255));
	ui->Pref_a_Details->setPalette(palette);

	// These are the two regular expressions
	QRegExpValidator *valid_pref_D = new QRegExpValidator(QRegExp("^([%][ATDdSq]){1}(/[%][ATDdSq]){0,}$"), this);
	ui->Pref_D_Details->setValidator(valid_pref_D);
	QRegExpValidator *valid_pref_a = new QRegExpValidator(QRegExp("^([%][ATDdSq]){0,1}(/[%][ATDdSq]){0,}$"), this);
	ui->Pref_a_Details->setValidator(valid_pref_a);

	// Initialise the directories database
	directories = QSqlDatabase::addDatabase("QSQLITE","directories");
	directories.setDatabaseName(QDir::homePath() + "/.RRipper/directories.db");
	directories.setHostName("localhost");
	directories.setUserName("rripper");
	directories.setPassword("");

	// Lets open the directory database
	if(!directories.open("rripper","")) {
		QMessageBox::critical(this,ERR,ERRT + "\n\nLoadRRipperInterfaceDBdirectories: " + directories.lastError().text());
	}

	// And as well the stations database
	stations = QSqlDatabase::addDatabase("QSQLITE");
	stations.setDatabaseName(QDir::homePath() + "/.RRipper/stations.db");
	stations.setHostName("localhost");
	stations.setUserName("rripper");
	stations.setPassword("");

	// Same as with the other database: It's the easiest way to make sure the table exists :)
	if(!stations.open()) {
		QMessageBox::critical(this,ERR,ERRT + "\n\nLoadRRipperInterfaceOpenDatabase: " + stations.lastError().text());
	}


	// Ok, load the station data now...
	LoadStationData();
	// ... and select the first row
	ui->list->selectRow(0);

	// Please load as well the directorie list (as available)
	DisplayStations();

	// Finally load the first row now
	SelectEntry(0,1);

	connect(lock,SIGNAL(timeout()),this, SLOT(locked()));

	// The connects for the station list
	connect(ui->list, SIGNAL(currentCellChanged(int,int,int,int)), this, SLOT(SelectEntry(int, int)));
	connect(cornerButton, SIGNAL(clicked()), this, SLOT(ResetStationListSorting()));

	// Changes in the global preferences
	connect(ui->Global_TrayIcon, SIGNAL(clicked()), this, SLOT(SaveGlobalPreferences()));
	connect(ui->Global_Matching, SIGNAL(activated(QString)), this, SLOT(SaveGlobalPreferences()));
	connect(ui->Global_Language, SIGNAL(activated(QString)), this, SLOT(SaveGlobalPreferences()));
	connect(ui->Lyric_AutoFetch, SIGNAL(clicked()), this, SLOT(SaveGlobalPreferences()));
	connect(ui->Global_FirstLyricsSource, SIGNAL(activated(QString)), this, SLOT(SaveGlobalPreferences()));
	connect(ui->Global_FirstLyricsSource, SIGNAL(activated(QString)), this, SLOT(FirstLyricsSourceChanged()));
	connect(ui->Global_DefaultLocationChoose, SIGNAL(clicked()), this, SLOT(SelectDefaultLocation()));
	connect(ui->Global_MPlayerPathChoose, SIGNAL(clicked()), this, SLOT(SelectMPlayerLocation()));
	connect(ui->Global_StreamRipperPathChoose, SIGNAL(clicked()), this, SLOT(SelectStreamripperLocation()));

	// The menu/toolbar functions
	connect(ui->New_Stream, SIGNAL(triggered()), this, SLOT(AddNewStation()));
	connect(ui->Quit, SIGNAL(triggered()), this, SLOT(QuitProgram()));
	connect(closeWindow, SIGNAL(triggered()), this, SLOT(close()));
	connect(sel_delete,SIGNAL(triggered()), this, SLOT(DeleteStation()));
	connect(sel_record, SIGNAL(triggered()), this, SLOT(Sel_StartRecording()));
	connect(ui->Details_Record, SIGNAL(clicked()), this, SLOT(Sel_StartRecording()));
	connect(all_record, SIGNAL(triggered()), this, SLOT(All_StartRecording()));
	connect(all_stop, SIGNAL(triggered()), this, SLOT(All_StopRecording()));
	connect(sel_play, SIGNAL(triggered()), this, SLOT(StartStopPlaying()));
	connect(ui->Details_Play, SIGNAL(clicked()), this, SLOT(StartStopPlaying()));
	connect(sel_stopAfterSong, SIGNAL(triggered()), this, SLOT(Sel_StopAfterSong()));
	connect(all_stopAfterSong, SIGNAL(triggered()), this, SLOT(All_StopAfterSong()));
	connect(ui->Kill_all_internal_processes, SIGNAL(triggered()), this, SLOT(KillAllInternalProcesses()));
	connect(ui->Kill_all_external_processes, SIGNAL(triggered()), this, SLOT(KillAllExternalProcesses()));
	connect(ui->About_RRipper, SIGNAL(triggered()), this, SLOT(AboutRRipper()));

	// The functions for the Details tab right site
	connect(ui->Details_SwapArtistTitle, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Details_LocationChoose, SIGNAL(clicked()), this, SLOT(SelectSavingLocation()));
	connect(ui->Details_StreamNameEdit, SIGNAL(textEdited(QString)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Details_URLEdit, SIGNAL(textEdited(QString)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Details_LocationEdit, SIGNAL(textEdited(QString)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Details_IntelligentPlayback, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Details_NormalPlayback, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));

	// The playback output of mplayer for analysing it
	connect(mplayer, SIGNAL(readyRead()), this, SLOT(GetMplayerOutput()));
	connect(streamripperforplaying, SIGNAL(readyRead()), this, SLOT(GetStreamripperForPlayingOutput()));

	// Not all Streamripper-Preferences go well together -> check
	connect(ui->Pref_D, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_s, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_a, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_l, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_M, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_i, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_wid3v1, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));
	connect(ui->Pref_woid3v2, SIGNAL(clicked()), this, SLOT(ValidateStreamripperPrefs()));

	// Connect the streamripper preferences fields
	connect(ui->Pref_D_Details, SIGNAL(textEdited(QString)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_r, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_a_Details, SIGNAL(textEdited(QString)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_o, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_o_Details, SIGNAL(activated(QString)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_c, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_l_Details, SIGNAL(valueChanged(int)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_M_Details, SIGNAL(valueChanged(int)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_q, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_q_Details, SIGNAL(valueChanged(int)), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_u, SIGNAL(clicked()), this, SLOT(UpdateStationsDatabase()));
	connect(ui->Pref_u_Details, SIGNAL(textEdited(QString)), this, SLOT(UpdateStationsDatabase()));

	connect(ui->Pref_D_Details, SIGNAL(textChanged(QString)), this, SLOT(RegExpPrefFields()));
	connect(ui->Pref_a_Details, SIGNAL(textChanged(QString)), this, SLOT(RegExpPrefFields()));

	connect(ui->Pref_D, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));
	connect(ui->Pref_a, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));
	connect(ui->Pref_o, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));
	connect(ui->Pref_l, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));
	connect(ui->Pref_M, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));
	connect(ui->Pref_q, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));
	connect(ui->Pref_u, SIGNAL(clicked()), this, SLOT(DisEnablePrefDetailsFields()));

	connect(ui->Pref_a, SIGNAL(clicked()), this, SLOT(RegExpPrefFields()));
	connect(ui->Pref_D, SIGNAL(clicked()), this, SLOT(RegExpPrefFields()));

	// Connect the directories SIGNALs with the functions
	connect(ui->Directories_Directory, SIGNAL(currentIndexChanged(QString)), this, SLOT(DirectoryChanged(QString)));
	connect(ui->Directories_list, SIGNAL(itemExpanded(QTreeWidgetItem*)), this, SLOT(DirectoriesItemExpanded(QTreeWidgetItem*)));
	connect(ui->Directories_list, SIGNAL(itemCollapsed(QTreeWidgetItem*)), this, SLOT(DirectoriesItemCollapsed(QTreeWidgetItem*)));
	connect(ui->Directories_list, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(AddStationToList(QTreeWidgetItem*)));
	connect(ui->Directories_FilterGenres, SIGNAL(textEdited(QString)), this, SLOT(DisplayStations()));
	connect(ui->Directories_FilterStations, SIGNAL(textEdited(QString)), this, SLOT(DisplayStations()));
	connect(ui->Directories_FilterBitrate, SIGNAL(valueChanged(int)), this, SLOT(DisplayStations()));
	connect(ui->Directories_allCollapse, SIGNAL(clicked()), this, SLOT(DirectoryAllCollapse()));
	connect(ui->Directories_Download, SIGNAL(clicked()), this, SLOT(RequestShoutcastGenres()));
	connect(ui->Directories_FilterBitrate, SIGNAL(valueChanged(int)), ui->Directories_FilterBitrateDisplay, SLOT(setValue(int)));
	connect(ui->Directories_FilterBitrateDisplay, SIGNAL(valueChanged(int)), ui->Directories_FilterBitrate, SLOT(setValue(int)));

	requestshoutcastgenres = new QNetworkAccessManager(this);		// Get the Shoutcast genres
	connect(requestshoutcastgenres, SIGNAL(finished(QNetworkReply*)), this, SLOT(AnalyseShoutcastGenres(QNetworkReply*)));

	requestshoutcaststations = new QNetworkAccessManager(this);
	connect(requestshoutcaststations, SIGNAL(finished(QNetworkReply*)), this, SLOT(AnalyseShoutcastStations(QNetworkReply*)));

	requesticecastdata = new QNetworkAccessManager(this);
	connect(requesticecastdata, SIGNAL(finished(QNetworkReply*)), this, SLOT(AnalyseIcecastData(QNetworkReply*)));

	// Connect the Lyrics functions
	connect(ui->Lyric_Fetch, SIGNAL(clicked()), this, SLOT(RequestLyrics()));
	connect(ui->Lyric_Reset, SIGNAL(clicked()), this, SLOT(ResetLyricFields()));
	connect(ui->Lyric_AutoFetch, SIGNAL(clicked()), this, SLOT(LyricAutoFetch()));

	requestlyrics = new QNetworkAccessManager(this);
	connect(requestlyrics, SIGNAL(finished(QNetworkReply*)), this, SLOT(AnalyseLyrics(QNetworkReply*)));


	connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));

	connect(toggleMainwindow, SIGNAL(triggered()), this, SLOT(showHideMainwindow()));
	connect(about, SIGNAL(triggered()), this, SLOT(AboutRRipper()));
	connect(quitProgram, SIGNAL(triggered()), this, SLOT(QuitProgram()));
	connect(ui->Global_TrayIcon, SIGNAL(clicked()), this, SLOT(showHideTrayIcon()));

	if(ui->Global_TrayIcon->isChecked()) {
		trayIcon->show();
	}


	connect(ui->Lyrics_External, SIGNAL(clicked()), this, SLOT(showLyricsExternal()));

	connect(volume,SIGNAL(valueChanged(int)), this, SLOT(volumeChanged()));
	connect(volumeMute, SIGNAL(toggled(bool)), this, SLOT(volumeMuted()));

}


void MainWindow::state(int type, QString out) {

	if(type == 1) {
		std::cout << "[INFO] " << out.toStdString() << std::endl;
	} else if(type == 2) {
		std::cout << "[ERROR] " << out.toStdString() << std::endl;
	} else if(type == 3) {
		std::cout << std::endl << ">>> [CRITICAL ERROR] <<<" << std::endl << out.toStdString() << std::endl << std::endl;
	} else if(type == 4) {
		std::cout << "[WARNING] " << out.toStdString() << std::endl;
	}

}


void MainWindow::showLyricsExternal() {

	external->show();

}


void MainWindow::createTrayIcon() {

		trayIcon = new QSystemTrayIcon(this);
		QIcon icon = QIcon(":/images/Logo_100.png");
		trayIcon->setIcon(icon);
		trayIcon->setToolTip(tr("RRipper - Radio Ripper and Player"));

		trayIconMenu = new QMenu(this);

		toggleMainwindow = new QAction("Hide/Show Window.",this);
		trayIconMenu->addAction(toggleMainwindow);

		about = new QAction(QIcon(":/images/AboutRRipper.png"),"About RRipper.",this);
		trayIconMenu->addAction(about);

		trayIconMenu->addSeparator();

		quitProgram = new QAction(QIcon(":/images/EndProgram.png"),"Quit RRipper.",this);
		trayIconMenu->addAction(quitProgram);

		trayIcon->setContextMenu(trayIconMenu);

}


void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason) {

	switch (reason) {
		case QSystemTrayIcon::Trigger:
			showHideMainwindow();
			break;
		default:
			;
	}


}


void MainWindow::showHideMainwindow() {

	if(this->isVisible()) {
		if(this->isMaximized()) {
			windowMaximised = 1;
		} else {
			windowMaximised = 0;
		}
		external->hide();
		this->hide();
		state(1,"Minimize RRipper to Tray");
	} else {
		if(windowMaximised == 1) {
			this->showMaximized();
		} else {
			this->show();
		}
		state(1,"Show RRipper");
	}

}


void MainWindow::showHideTrayIcon() {

	if(ui->Global_TrayIcon->isChecked()) {
		trayIcon->show();
	} else {
		trayIcon->hide();
	}

}


void MainWindow::volumeMuted() {

	if(volumeBeforeMuting == -1) {
		volumeBeforeMuting = volume->value();
		volume->setValue(0);
	} else {
		volume->setValue(volumeBeforeMuting);
		volumeBeforeMuting = -1;
	}

}


void MainWindow::volumeChanged() {

	QByteArray x("volume ");
	x.append(QString("%1 1\n").arg(volume->value()));
	mplayer->write(x);
	QString value = QString("%1").arg(volume->value());
	while(value.size() < 3) {
		value = QString("0") + value;
	}
	volume->setToolTip(QString("%1 %").arg(volume->value()));
	volume_value->setText(QString("%1 %").arg(value));

	if(volume->value() != 0 && volumeBeforeMuting != -1) {
		volumeBeforeMuting = -1;
		volumeMute->setChecked(false);
	}

}


void MainWindow::showMessage(int type, QString title, QString body) {
	
	QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::MessageIcon(type);
	trayIcon->showMessage(title, body, icon, 5000);

}

// Every 2 seconds a file is updated to show that an instance of RRipper is running
void MainWindow::locked() {

	QFile file(QDir::homePath() + "/.RRipper/.locked");

	if(file.open(QIODevice::ReadWrite)) {
		QTextStream timestamp(&file);

		timestamp << QDateTime::currentDateTime().toTime_t();

		file.close();

	}

	QFile commands(QDir::homePath() + "/.RRipper/dothat");

	if(commands.open(QIODevice::ReadWrite)) {

		QTextStream in(&commands);
		QStringList lines = in.readAll().split("\n");
		for(int i = 0; i < lines.size(); ++i) {
			if(lines.at(i) == "--show") {
				if(!this->isVisible()) {
					if(windowMaximised == 1) {
						this->showMaximized();
					} else {
						this->show();
					}
				} else {
					this->raise();
					this->setFocus();
				}
			} else if(lines.at(i) == "--play") {
				StartStopPlaying();
			}
		}
	}

	commands.remove();

}


// Create the submenus/toolbar (because of submenus not possible to do in designer (BUG))
void MainWindow::CreateToolbar() {

	state(1,"Creating Toolbar");

	// First submenu
	QMenu *sel = new QMenu;
	sel_delete= new QAction(QIcon(":/images/StreamDelete.png"),tr("&Delete"),this);
	sel_record = new QAction(QIcon(":/images/StreamStartRecording.png"),tr("Start/Stop &Recording"),this);
	sel_play = new QAction(QIcon(":/images/StreamPlay.png"),tr("Start/Stop &Playing"),this);
	sel_stopAfterSong = new QAction(QIcon(":/images/StreamStopRecordingAfterSong.png"),tr("Stop &after song"),this);
	QList<QKeySequence> sel_delete_shortcuts;
	sel_delete_shortcuts << QKeySequence(tr("Ctrl+D")) << QKeySequence(tr("Del"));
	QKeySequence sel_record_shortcut = QKeySequence(tr("Ctrl+R"));
	QList<QKeySequence> sel_play_shortcuts;
	sel_play_shortcuts << QKeySequence(tr("Ctrl+P")) << QKeySequence(tr("Space"));
	QKeySequence sel_stopAfterSong_shortcut = QKeySequence(tr("Ctrl+S"));
	sel_delete->setShortcuts(sel_delete_shortcuts);
	sel_record->setShortcut(sel_record_shortcut);
	sel_play->setShortcuts(sel_play_shortcuts);
	sel_stopAfterSong->setShortcut(sel_stopAfterSong_shortcut);
	sel_delete->setToolTip(tr("Delete selected stream (irreversible)."));
	sel_delete->setStatusTip(tr("Delete selected stream"));
	sel_record->setToolTip(tr("Start/Stop Recording of selected stream."));
	sel_record->setStatusTip(tr("Start/Stop Recording of selected stream"));
	sel_play->setToolTip(tr("Play/Stop selected stream."));
	sel_play->setStatusTip(tr("Play/Stop selected stream"));
	sel_stopAfterSong->setToolTip(tr("Stop the current recording after this song."));
	sel_stopAfterSong->setStatusTip(tr("Stop the current recording after this song"));
	sel->addAction(sel_delete);
	sel->addAction(sel_record);
	sel->addAction(sel_play);
	sel->addAction(sel_stopAfterSong);
	ui->Selected->setMenu(sel);

	// Second submenu
	QMenu *all = new QMenu;
	all_record = new QAction(QIcon(":/images/AllStreamStartRecording.png"),tr("&Record"),this);
	all_stop = new QAction(QIcon(":/images/AllStreamStopRecording.png"),tr("&Stop"),this);
	all_stopAfterSong = new QAction(QIcon(":/images/AllStreamStopRecordingAfterSong.png"),tr("Stop &after song"),this);
	QKeySequence all_record_shortcut = QKeySequence(tr("Ctrl+A"));
	QKeySequence all_stop_shortcut = QKeySequence(tr("Ctrl+B"));
	QKeySequence all_stopAfterSong_shortcut = QKeySequence(tr("Ctrl+H"));
	all_record->setShortcut(all_record_shortcut);
	all_stop->setShortcut(all_stop_shortcut);
	all_stopAfterSong->setShortcut(all_stopAfterSong_shortcut);
	all_record->setToolTip(tr("Record all streams in the list."));
	all_record->setStatusTip(tr("Record all streams in the list"));
	all_stop->setToolTip(tr("Stop all running recordings."));
	all_stop->setStatusTip(tr("Stop all running recordings"));
	all_stopAfterSong->setToolTip(tr("Stop all running recordings after current song."));
	all_stopAfterSong->setStatusTip(tr("Stop all running recordings after current song"));
	all->addAction(all_record);
	all->addAction(all_stop);
	all->addAction(all_stopAfterSong);
	ui->All->setMenu(all);


	save_song = new QPushButton(tr("Save"));
	save_song->setEnabled(false);
	save_song->setToolTip("Save currently played song.");
	save_song->setStatusTip("Save currently played song.");
	save_song->setStyleSheet("font-size: 8pt;");
	save_song->setCheckable(true);

	volume = new QSlider;
	volume->setMinimum(0);
	volume->setMaximum(100);
	volume->setOrientation(Qt::Horizontal);
	volume->setSliderPosition(100);
	volume->setToolTip("100 %");

	volume_value = new QLabel("100 %");
	volume_value->setToolTip(tr("Current volume"));

	volumeMute = new QPushButton("M");
	volumeMute->setFixedSize(20,20);
	volumeMute->setCheckable(true);
	volumeMute->setToolTip(tr("Mute"));

	QHBoxLayout *hbox = new QHBoxLayout;

	hbox->addStretch(50);
	hbox->addWidget(volumeMute);
	hbox->addWidget(volume);
	hbox->addWidget(volume_value);
	hbox->addStretch(5);
	hbox->addWidget(save_song);


	QWidget *widget = new QWidget;
	widget->setLayout(hbox);

	// Toolbar
	closeWindow = new QAction(QIcon(":/images/EndProgram.png"),"Close MainWindow",this);
	closeWindow->setToolTip(tr("Close Mainwindow (minimises in system tray)"));
	closeWindow->setStatusTip(tr("Close Mainwindow (minimises in system tray)"));
	ui->toolBar->addAction(closeWindow);
	ui->toolBar->addSeparator();
	ui->toolBar->addAction(ui->New_Stream);
	ui->toolBar->addAction(sel_delete);
	ui->toolBar->addSeparator();
	ui->toolBar->addAction(sel_record);
	ui->toolBar->addAction(sel_play);
	ui->toolBar->addAction(sel_stopAfterSong);
	ui->toolBar->addSeparator();
	ui->toolBar->addAction(all_record);
	ui->toolBar->addAction(all_stop);
	ui->toolBar->addAction(all_stopAfterSong);
	ui->toolBar->addSeparator();
	ui->toolBar->addAction(ui->Kill_all_internal_processes);
	ui->toolBar->addAction(ui->Kill_all_external_processes);
	ui->toolBar->addWidget(widget);


}


// This functions is called when the window is resized to adjust the headers of the station list
void MainWindow::ResizeColumns() {

	state(1,"Resizing Columns");

	// The table has 7 columns, but the last one contains the code is not important for the user -> hidden
	ui->list->setColumnCount(7);
	ui->list->hideColumn(6);

	// That's the actual size of the table view
	int size = ui->list->width();

	// These are the procents of width of each column (It's in total only 99%, because otherwise it could be maybe a pixel to wide (due to calculations restrictions). So it's only 99% and the name-column has the preference 'stretch')
	rowone = 0.04;
	rowtwo = 0.18;
	rowthree = 0.25;
	rowfour = 0.22;
	rowfive = 0.20;
	rowsix = 0.10;

	// Resize the columns
	ui->list->setColumnWidth(0,rowone*size-1);
	ui->list->setColumnWidth(1,rowtwo*size-1);
	ui->list->setColumnWidth(2,rowthree*size-1);
	ui->list->setColumnWidth(3,rowfour*size-1);
	ui->list->setColumnWidth(4,rowfive*size-1);
	ui->list->setColumnWidth(5,rowsix*size-1);

	// And set up the headers. I've made them fix, the third one ('name') is reponsible, that the whole space is used
	ui->list->horizontalHeader()->setResizeMode(0,QHeaderView::Fixed);
	ui->list->horizontalHeader()->setResizeMode(1,QHeaderView::Fixed);
	ui->list->horizontalHeader()->setResizeMode(2,QHeaderView::Stretch);
	ui->list->horizontalHeader()->setResizeMode(3,QHeaderView::Fixed);
	ui->list->horizontalHeader()->setResizeMode(4,QHeaderView::Fixed);
	ui->list->horizontalHeader()->setResizeMode(5,QHeaderView::Fixed);


}


// This function is called by the corner left button of the station list table view, to reset the sorting
void MainWindow::ResetStationListSorting() {

	state(1,"Sorting of station list reset");

	ui->list->horizontalHeader()->setSortIndicator(-1,Qt::AscendingOrder);

}


// This function makes it so muchb easier to catch SQL errors and output them on the konsole
void MainWindow::SQLERR(QSqlDatabase database,QSqlQuery query, QString where) {

	if(query.lastError().text().trimmed() != "") {
		state(2,"in SQL QUERY in '" + where + "'; SQL error message: '" + query.lastError().text() + "'");
	}

	if(database.lastError().text().trimmed() != "") {
		state(2,"in DATABASE OPERATION in '" + where + "'; SQL error message: '" + database.lastError().text() + "'");
	}

}


// This creates the string containing the streamripper preferences for the station with the code 'code'
QStringList MainWindow::CreateStreamripperPrefString(QString code) {

	QSqlQuery query;

	query.exec("SELECT * FROM details WHERE code LIKE '" + code + "'");
	query.next();

	// This contains all the preferences
	QStringList arguments;

	QRegExpValidator *valid_pref_D = new QRegExpValidator(QRegExp("^([%][ATDdSq]){1}(/[%][ATDdSq]){0,}$"), this);
	QString text = query.value(query.record().indexOf("D")).toString();
	int pos = 0;
	int D_used = 0;
	if(valid_pref_D->validate(text,pos) == 2) {
		arguments << "-D" << query.value(query.record().indexOf("location")).toString() + text;
		D_used = 1;
	} else {
		state(4,"Streamripper preference '-D' invalid. Reset to default saving pattern.");
		arguments << "-d" << query.value(query.record().indexOf("location")).toString();
	}

	if(D_used == 0) {
		if(query.value(query.record().indexOf("a_checked")).toString() == "1") {
			arguments << "-a";
			QRegExpValidator *valid_pref_a = new QRegExpValidator(QRegExp("^([%][ATDdSq]){0,1}(/[%][ATDdSq]){0,}$"), this);
			QString text = query.value(query.record().indexOf("a_text")).toString();
			int pos = 0;
			if(valid_pref_a->validate(text,pos) == 2) {
				arguments << text;
			} else {
				state(4,"Streamripper preference '-a' invalid. Ignored.");
			}

		}
	}

	if(query.value(query.record().indexOf("s")).toString() == "1") {
		arguments << "-s";
	}

	if(query.value(query.record().indexOf("r")).toString() == "1") {
		arguments << "-r";
	}

	if(query.value(query.record().indexOf("o")).toString() != "-1") {
		QStringList o_options;
		o_options << "always" << "never" << "larger" << "version";
		bool ok;
		QString o = o_options.at(query.value(query.record().indexOf("o")).toInt(&ok));
		if(ok == true) {
			arguments << "-o" << o;
		} else {
			state(4,"Error in reading Streamripper preference '-o'. Ignored.");
		}
	}

	if(query.value(query.record().indexOf("c")).toString() == "1") {
		arguments << "-c";
	}

	int l_used = 0;
	if(query.value(query.record().indexOf("l")).toString() != "0") {
		arguments << "-l" << query.value(query.record().indexOf("l")).toString();
		l_used = 1;
	}

	if(l_used == 0) {
		if(query.value(query.record().indexOf("M")).toString() != "0") {
			arguments << "-M" << query.value(query.record().indexOf("M")).toString();
		}
	}

	if(query.value(query.record().indexOf("q")).toString() != "0") {
		arguments << "-q" << query.value(query.record().indexOf("q")).toString();
	}

	if(query.value(query.record().indexOf("u")) != "") {
		arguments << "-u" << query.value(query.record().indexOf("u")).toString();
	}

	int i_used = 0;
	if(query.value(query.record().indexOf("i")).toString() == "1") {
		arguments << "-i";
		i_used = 1;
	}

	if(i_used == 0) {

		if(query.value(query.record().indexOf("wid3v1")).toString() == "1") {
			arguments << "--with-id3v1";
		}

		if(query.value(query.record().indexOf("woid3v2")).toString() == "1") {
			arguments << "--without-id3v2";
		}

	}

	SQLERR(stations,query,"CreateStreamripperPrefString");

	query.clear();

	return arguments;

}


// When an intelligent playback is started, the tempporary directory is first deleted (recursive!)
void MainWindow::DeleteDirectoryRecursive(QString directory) {

	QDir dir(directory);
	dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);

	QDir dirFile(directory);
	dirFile.setFilter(QDir::Files);

	QFileInfoList list = dir.entryInfoList();
	QFileInfoList listFile = dirFile.entryInfoList();

	if(list.size() > 0) {
		for(int i = 0; i < list.size(); ++i) {
			QFileInfo info = list.at(i);
			DeleteDirectoryRecursive(info.absoluteFilePath());
		}
		for(int i = 0; i < listFile.size(); ++i) {
			QFile file(listFile.at(i).absoluteFilePath());
			file.remove();
		}
		dir.rmdir(directory);
	} else {

		for(int i = 0; i < listFile.size(); ++i) {
			QFile file(listFile.at(i).absoluteFilePath());
			file.remove();
		}
		dir.rmdir(directory);
	}

}


// Load the global preferences
void MainWindow::LoadGlobalPreferences() {

	state(1,"Loading global preferences");

	// That's the default setup
	ui->Global_TrayIcon->setChecked(true);
	ui->Global_Matching->setCurrentIndex(2);
	ui->Global_DefaultLocation->setText(QDir::homePath() + "/");
	ui->Global_MPlayerPath->setText("/usr/bin/mplayer");
	ui->Global_StreamRipperPath->setText("/usr/bin/streamripper");
	ui->Global_Language->setCurrentIndex(0);
	ui->Lyric_AutoFetch->setChecked(true);
	ui->Global_FirstLyricsSource->setCurrentIndex(0);

	QFile file(QDir::homePath() + "/.RRipper/preferences");

	if(!file.open(QIODevice::ReadOnly)) {
		QMessageBox::critical(this,ERR,ERRT + "\n\nGlobalPreferencesCannotOpen: LoadGlobalPreferences");
	} else {

		QTextStream read_all(&file);
		QStringList preferences = read_all.readAll().split("\n");

		// Analysing evry line
		for(int i = 0; i < preferences.size(); ++i) {

			QStringList temp = preferences.at(i).split("=");
			QString left = temp.at(0);
			QString right;
			if(temp.size() == 2) {
				right = temp.at(1);
			} else {
				right = "";
			}

			if(left == "TrayIcon") {
				bool ok;
				ui->Global_TrayIcon->setChecked(right.toInt(&ok,10));
			} else if(left == "NotMatching") {
				bool ok;
				ui->Global_Matching->setCurrentIndex(right.toInt(&ok,10));
			} else if(left == "DefaultLocation") {
				ui->Global_DefaultLocation->setText(right);
			} else if(left == "MPlayer") {
				ui->Global_MPlayerPath->setText(right);
			} else if(left == "Streamripper") {
				ui->Global_StreamRipperPath->setText(right);
			} else if(left == "Language") {
				bool ok;
				ui->Global_Language->setCurrentIndex(right.toInt(&ok,10));
			} else if(left == "LyricAutoFetch") {
				bool ok;
				ui->Lyric_AutoFetch->setChecked(right.toInt(&ok,10));
			} else if(left == "LyricsFirstSource") {
				bool ok;
				int source = 0;
				source = right.toInt(&ok,10);
				ui->Global_FirstLyricsSource->setCurrentIndex(source);
				if(source == 0) {
					lyricsSource = 0;
				} else {
					lyricsSource = 2;
				}
			}

		}


	}

}


// Load the station data completely new (this function is only called at the beginning!)
void MainWindow::LoadStationData() {

	state(1,"Loading Stations");


	// Get all the stations from the database
	QSqlQuery data;
	data.exec("SELECT code,name,playback FROM details ORDER BY name");

	int row = 0;

	while(data.next()) {

		ui->list->setRowCount(row+1);

		// The name is the status tip of each cell
		QString name = data.value(data.record().indexOf("name")).toString();

		QTableWidgetItem *item = new QTableWidgetItem("");
		item->setIcon(QIcon(":/images/StreamStopRecording.png"));
		ui->list->setItem(row,0,item);

		item = new QTableWidgetItem(status_notconnected);
		item->setStatusTip(name);
		ui->list->setItem(row,1,item);

		item = new QTableWidgetItem(name);
		item->setStatusTip(name);
		ui->list->setItem(row,2,item);

		item = new QTableWidgetItem("[" + id3_artist + "]");
		item->setStatusTip(name);
		ui->list->setItem(row,3,item);

		item = new QTableWidgetItem("[" + id3_title + "]");
		item->setStatusTip(name);
		ui->list->setItem(row,4,item);


		item = new QTableWidgetItem("---");
		item->setStatusTip(name);
		ui->list->setItem(row,5,item);

		item = new QTableWidgetItem(data.value(data.record().indexOf("code")).toString());
		item->setStatusTip(name);
		ui->list->setItem(row,6,item);

		// Set as well up the temporary values
		values[data.value(data.record().indexOf("code")).toString()]["localhost"] = "---";
		values[data.value(data.record().indexOf("code")).toString()]["recording"] = "0";
		values[data.value(data.record().indexOf("code")).toString()]["playing"] = "0";
		values[data.value(data.record().indexOf("code")).toString()]["playback"] = data.value(data.record().indexOf("playback")).toString();
		values[data.value(data.record().indexOf("code")).toString()]["stopaftersong"] = "0";
		values[data.value(data.record().indexOf("code")).toString()]["startrecording"] = "0";
		values[data.value(data.record().indexOf("code")).toString()]["stoprecording"] = "0";
		values[data.value(data.record().indexOf("code")).toString()]["timeout"] = "0";

		++row;

	}

	SQLERR(stations,data,"LoadStationData");

	data.clear();

}


// After selecting one of the entries (a = row; b = column)
void MainWindow::SelectEntry(int a, int b) {

	// This has only the function to avoid a warning while compiling. In fact the column is not important...
	b = b*1;

	// The code of the currently selected station
	QString code = ui->list->item(a,6)->text();

	// Select all the connected data
	QSqlQuery data;
	data.exec("SELECT * FROM details WHERE code LIKE '" + code + "'");

	SQLERR(stations,data,"SelectEntry 1");

	data.next();

	// And enter it in the details tab
	bool ok;
	ui->Details_SwapArtistTitle->setChecked(data.value(data.record().indexOf("swapartisttitle")).toInt(&ok));

	if(data.value(data.record().indexOf("playback")).toString() == "intelligent") {
		ui->Details_IntelligentPlayback->setChecked(true);
	} else {
		ui->Details_NormalPlayback->setChecked(true);
	}

	if(played == data.value(data.record().indexOf("code")).toString()) {
		ui->Details_IntelligentPlayback->setEnabled(false);
		ui->Details_NormalPlayback->setEnabled(false);
		ui->Details_PlaybackLabel->setEnabled(false);
		sel_play->setIcon(QIcon(":/images/StreamStop.png"));
		ui->Details_Play->setIcon(QIcon(":/images/StreamStop.png"));
	} else {
		if(values[code]["recording"] == "0") {
			ui->Details_IntelligentPlayback->setEnabled(true);
		} else {
			ui->Details_IntelligentPlayback->setEnabled(false);
		}
		ui->Details_NormalPlayback->setEnabled(true);
		ui->Details_PlaybackLabel->setEnabled(true);
		sel_play->setIcon(QIcon(":/images/StreamPlay.png"));
		ui->Details_Play->setIcon(QIcon(":/images/StreamPlay.png"));
	}

	ui->Details_StreamName->setText("<p align=\"center\" style=\"font-size: 12pt\"><b>" + data.value(data.record().indexOf("name")).toString() + "</b></p>");
	ui->Details_ServerName->setText(data.value(data.record().indexOf("server")).toString());
	ui->Details_Bitrate->setText(data.value(data.record().indexOf("bitrate")).toString());
	if(data.value(data.record().indexOf("codec")).toString() != "") {
		ui->Details_Bitrate->setText(ui->Details_Bitrate->text() + " (" + data.value(data.record().indexOf("codec")).toString() + ")");
	}
	ui->Details_MetaInterval->setText(data.value(data.record().indexOf("metainterval")).toString());

	ui->Details_StreamNameEdit->setText(data.value(data.record().indexOf("name")).toString());
	ui->Details_URLEdit->setText(data.value(data.record().indexOf("url")).toString());
	ui->Details_LocationEdit->setText(data.value(data.record().indexOf("location")).toString());

	// If the station is not recorded and no technical details are available, the mplayer specification is used
	if(played == code) {
		if((ui->Details_ServerName->text() == "" || ui->Details_ServerName->text() == "---") && played_values["server"] != "") {
			ui->Details_ServerName->setText(played_values["server"]);
		}
		if((ui->Details_Bitrate->text() == "" || ui->Details_Bitrate->text() == "---") && played_values["bitrate"] != "") {
			ui->Details_Bitrate->setText(played_values["bitrate"]);
		}
		if(!ui->Details_Bitrate->text().endsWith(")") && played_values["codec"] != "") {
			ui->Details_Bitrate->setText(ui->Details_Bitrate->text() + " (" + played_values["codec"] + ")");
		}
	}

	// The artist (if there is information available)
	QString text;
	if(ui->list->item(a,3)->text() != "" || ui->list->item(a,4)->text() != "") {

		QString artist = ui->list->item(a,3)->text().trimmed();
		if(artist.startsWith("[") && artist.endsWith("]")) {
			artist = artist.remove(0,1);
			artist = artist.remove(artist.length()-1,1);
		}
		QString title = ui->list->item(a,4)->text().trimmed();
		if(title.startsWith("[") && title.endsWith("]")) {
			title = title.remove(0,1);
			title = title.remove(title.length()-1,1);
		}

		text = "<p align=\"center\">";
		if(artist != "") {
			text += "<b>" + artist + "</b> - ";
		}
		text += title + "</p>";
	} else {
		text = "<p align=\"center\"><b>" + id3_artist + "</b> - " + id3_title + "</p>";
	}
	ui->Details_ArtistTitle->setText(text);


	if(values[code]["recording"] == "1") {
		ui->Preferences->setEnabled(false);
		ui->Details_URLEdit->setEnabled(false);
		ui->Details_LocationEdit->setEnabled(false);
		ui->Details_LocationChoose->setEnabled(false);
		sel_record->setIcon(QIcon(":/images/StreamStopRecording.png"));
		ui->Details_Record->setIcon(QIcon(":/images/StreamStopRecording.png"));

	} else {
		ui->Preferences->setEnabled(true);
		ui->Details_URLEdit->setEnabled(true);
		ui->Details_LocationEdit->setEnabled(true);
		ui->Details_LocationChoose->setEnabled(true);
		sel_record->setIcon(QIcon(":/images/StreamStartRecording.png"));
		ui->Details_Record->setIcon(QIcon(":/images/StreamStartRecording.png"));
	}


	// And of course, enter as well the data into the streamripper tab
	if(data.value(data.record().indexOf("D")).toString() != "") {
		ui->Pref_D->setChecked(true);
		ui->Pref_D_Details->setText(data.value(data.record().indexOf("D")).toString());
	} else {
		ui->Pref_D->setChecked(false);
		ui->Pref_D_Details->setText("");
	}

	ui->Pref_s->setChecked(data.value(data.record().indexOf("s")).toInt(&ok));
	ui->Pref_r->setChecked(data.value(data.record().indexOf("r")).toInt(&ok));

	ui->Pref_a->setChecked(data.value(data.record().indexOf("a_checked")).toInt(&ok));
	if(ui->Pref_a->isChecked()) {
		ui->Pref_a_Details->setText(data.value(data.record().indexOf("a_text")).toString());
	} else {
		ui->Pref_a_Details->setText("");
	}

	if(data.value(data.record().indexOf("o")).toString() != "-1") {
		ui->Pref_o->setChecked(true);
		ui->Pref_o_Details->setCurrentIndex(data.value(data.record().indexOf("o")).toInt(&ok));
	} else {
		ui->Pref_o->setChecked(false);
		ui->Pref_o_Details->setCurrentIndex(2);
	}

	ui->Pref_c->setChecked(data.value(data.record().indexOf("c")).toInt(&ok));

	ui->Pref_l->setChecked(data.value(data.record().indexOf("l")).toInt(&ok));
	if(ui->Pref_l->isChecked()) {
		ui->Pref_l_Details->setValue(data.value(data.record().indexOf("l")).toInt(&ok));
	} else {
		ui->Pref_l_Details->setValue(1);
	}

	ui->Pref_M->setChecked(data.value(data.record().indexOf("M")).toInt(&ok));
	if(ui->Pref_M->isChecked()) {
		ui->Pref_M_Details->setValue(data.value(data.record().indexOf("M")).toInt(&ok));
	} else {
		ui->Pref_M_Details->setValue(1);
	}

	ui->Pref_q->setChecked(data.value(data.record().indexOf("q")).toInt(&ok));
	if(ui->Pref_q->isChecked()) {
		ui->Pref_q_Details->setValue(data.value(data.record().indexOf("q")).toInt(&ok));
	} else {
		ui->Pref_q_Details->setValue(1);
	}

	if(data.value(data.record().indexOf("u")).toString() != "") {
		ui->Pref_u->setChecked(true);
		ui->Pref_u_Details->setText(data.value(data.record().indexOf("u")).toString());
	} else {
		ui->Pref_u->setChecked(false);
		ui->Pref_u_Details->setText("Streamripper/1.x");
	}

	ui->Pref_i->setChecked(data.value(data.record().indexOf("i")).toInt(&ok));
	ui->Pref_wid3v1->setChecked(data.value(data.record().indexOf("wid3v1")).toInt(&ok));
	ui->Pref_woid3v2->setChecked(data.value(data.record().indexOf("woid3v2")).toInt(&ok));

	SQLERR(stations,data,"SelectEntry 2");

	data.clear();

	// Dis-/Enable the preference fields for the additional information
	DisEnablePrefDetailsFields();
	// validate the fields for '-D' and '-a'
	RegExpPrefFields();

}


// Select a new saving location for the rips of the currently selected station
void MainWindow::SelectSavingLocation() {

	// Code of currently selected station
	QString code = ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text();

	// Opening a FileDialog for selecting the new directory
	QString new_location = QFileDialog::getExistingDirectory(this,tr("Select Saving Location"),ui->Details_LocationEdit->text());

	// If not set, add slash at the end (just to have conformity)
	if(!new_location.endsWith("/")) {
		new_location += "/";
	}

	// If there is an actual new selected location (no click on "Cancel")
	if(new_location.size() > 1) {

		// Insert...
		ui->Details_LocationEdit->setText(new_location);

		// .. and save in database

		QSqlQuery query;
		query.exec("UPDATE details SET location='" + new_location + "' WHERE code LIKE '" + code + "'");

		SQLERR(stations,query,"SelectSavingLocation");

		query.clear();

	}

}


// Some streamripper preferences don't go together -> check
void MainWindow::ValidateStreamripperPrefs() {

	QString code = ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text();

	QSqlQuery query;

	query.exec("SELECT D,s,a_checked,l,M,i,wid3v1,woid3v2 FROM details WHERE code LIKE '" + code + "'");

	SQLERR(stations,query,"ValidateStreamripperPrefs");

	query.next();

	if(query.value(query.record().indexOf("D")).toString() == "" && ui->Pref_D->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_a->setChecked(false);
		ui->Pref_s->setChecked(false);
	} else if(query.value(query.record().indexOf("s")).toString() == "0" && ui->Pref_s->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_D->setChecked(false);
	} else if(query.value(query.record().indexOf("a_checked")).toString() == "0" && ui->Pref_a->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_D->setChecked(false);
	}

	if(query.value(query.record().indexOf("l")).toString() == "0" && ui->Pref_l->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_M->setChecked(false);
	} else if(query.value(query.record().indexOf("M")).toString() == "0" && ui->Pref_M->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_l->setChecked(false);
	}

	if(query.value(query.record().indexOf("i")).toString() == "0" && ui->Pref_i->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_wid3v1->setChecked(false);
		ui->Pref_woid3v2->setChecked(false);
	} else if(query.value(query.record().indexOf("wid3v1")).toString() == "0" && ui->Pref_wid3v1->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_i->setChecked(false);
	} else if(query.value(query.record().indexOf("woid3v2")).toString() == "0" && ui->Pref_woid3v2->isChecked()) {
		SQLERR(stations,query,"ValidateStreamripperPrefs");
		ui->Pref_i->setChecked(false);
	}

	query.clear();

	UpdateStationsDatabase();

	DisEnablePrefDetailsFields();

	RegExpPrefFields();

}


// After every change of preferences/settings -> save it
void MainWindow::UpdateStationsDatabase() {

	// Save the currently ordering
	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();

	// ...and remove it (for being able to insert new data without having the problem of resorting after evry value; the sorting is added again at the end)
	ui->list->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);

	// Currently selected row and code
	int row = ui->list->selectedItems().at(1)->row();
	QString code = ui->list->item(row,6)->text();

	stations.transaction();

	QSqlQuery query;
	query.prepare("UPDATE details SET swapartisttitle=:swap,playback=:play,name=:name,url=:url,location=:location,D=:D,s=:s,r=:r,a_checked=:a_checked,a_text=:a_text,o=:o,c=:c,l=:l,M=:M,q=:q,u=:u,i=:i,wid3v1=:wid3v1,woid3v2=:woid3v2 WHERE code LIKE '" + code + "'");

	// Set the data  of the "Details" tab
	query.bindValue(":swap",QString("%1").arg(ui->Details_SwapArtistTitle->isChecked()));

	if(ui->Details_IntelligentPlayback->isChecked()) {
		query.bindValue(":play","intelligent");
		values[code]["playback"] = "intelligent";
	} else {
		query.bindValue(":play","normal");
		values[code]["playback"] = "normal";
	}
	query.bindValue(":name",ui->Details_StreamNameEdit->text());
	query.bindValue(":url",ui->Details_URLEdit->text());
	query.bindValue(":location",ui->Details_LocationEdit->text());

	// Get all the streamripper preferences and add them

	if(ui->Pref_D->isChecked()) {
		query.bindValue(":D",ui->Pref_D_Details->text());
	} else {
		query.bindValue(":D","");
	}

	query.bindValue(":s",QString("%1").arg(ui->Pref_s->isChecked()));
	query.bindValue(":r",QString("%1").arg(ui->Pref_r->isChecked()));

	query.bindValue(":a_checked",QString("%1").arg(ui->Pref_a->isChecked()));
	if(ui->Pref_a->isChecked()) {
		query.bindValue(":a_text",ui->Pref_a_Details->text());
	} else {
		query.bindValue(":a_text","");
	}

	if(ui->Pref_o->isChecked()) {
		query.bindValue(":o",QString("%1").arg(ui->Pref_o_Details->currentIndex()));
	} else {
		query.bindValue(":o","-1");
	}

	query.bindValue(":c",QString("%1").arg(ui->Pref_c->isChecked()));

	if(ui->Pref_l->isChecked()) {
		query.bindValue(":l",QString("%1").arg(ui->Pref_l_Details->value()));
	} else {
		query.bindValue(":l","0");
	}

	if(ui->Pref_M->isChecked()) {
		query.bindValue(":M",QString("%1").arg(ui->Pref_M_Details->value()));
	} else {
		query.bindValue(":M","0");
	}

	if(ui->Pref_q->isChecked()) {
		query.bindValue(":q",QString("%1").arg(ui->Pref_q_Details->value()));
	} else {
		query.bindValue(":q","0");
	}

	if(ui->Pref_u->isChecked()) {
		query.bindValue(":u",ui->Pref_u_Details->text());
	} else {
		query.bindValue(":u","");
	}

	query.bindValue(":i",QString("%1").arg(ui->Pref_i->isChecked()));
	query.bindValue(":wid3v1",QString("%1").arg(ui->Pref_wid3v1->isChecked()));
	query.bindValue(":woid3v2",QString("%1").arg(ui->Pref_woid3v2->isChecked()));


	query.exec();

	SQLERR(stations,query,"UpdateStationDatabase (exec)");

	stations.commit();

	SQLERR(stations,query,"UpdateStationDatabase (commit)");

	query.clear();


	ui->Details_StreamName->setText("<p align=\"center\" style=\"font-size: 12pt\"><b>" + ui->Details_StreamNameEdit->text() + "</b></p>");
	ui->list->item(row,2)->setText(ui->Details_StreamNameEdit->text());
	ui->list->item(row,2)->setStatusTip(ui->Details_StreamNameEdit->text());


	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}


}


// Save the global preferences to a file
void MainWindow::SaveGlobalPreferences() {

	QFile file(QDir::homePath() + "/.RRipper/preferences");
	if(!file.open(QIODevice::WriteOnly)) {
		QMessageBox::critical(this,tr("Error"),tr("Error saving Global Preferences. Couldn't open file '%1/.RRipper/preferences' for writing.").arg(QDir::homePath()));
	} else {

		QString content = "";
		content += "TrayIcon=" + QString("%1").arg(ui->Global_TrayIcon->isChecked());
		content += "\nNotMatching=" + QString("%1").arg(ui->Global_Matching->currentIndex());
		content += "\nDefaultLocation=" + ui->Global_DefaultLocation->text();
		content += "\nMPlayer=" + ui->Global_MPlayerPath->text();
		content += "\nStreamripper=" + ui->Global_StreamRipperPath->text();
		content += "\nLanguage=" + QString("%1").arg(ui->Global_Language->currentIndex());
		content += "\nLyricAutoFetch=" + QString("%1").arg(ui->Lyric_AutoFetch->isChecked());
		content += "\nLyricsFirstSource=" + QString("%1").arg(ui->Global_FirstLyricsSource->currentIndex());

		QTextStream out(&file);

		out << content;

		file.close();

	}

}


// Select the defaukt saving location a new stream gets
void MainWindow::SelectDefaultLocation() {

	QString directory = QFileDialog::getExistingDirectory(this,tr("Choose Default Saving Location..."),ui->Global_DefaultLocation->text());

	if(!directory.endsWith("/")) {
		directory = directory + "/";
	}

	if(directory.size() > 1) {
		ui->Global_DefaultLocation->setText(directory);
	}

	SaveGlobalPreferences();

}


// Select the executeable mplayer file
void MainWindow::SelectMPlayerLocation() {

	QString file = QFileDialog::getOpenFileName(this,tr("Select executeable MPlayer file..."),ui->Global_MPlayerPath->text());

	if(file != "") {
		ui->Global_MPlayerPath->setText(file);
	}

	SaveGlobalPreferences();

}


// Select the executeable streamripper file
void MainWindow::SelectStreamripperLocation() {

	QString file = QFileDialog::getOpenFileName(this,tr("Select executeable Streamripper file..."),ui->Global_StreamRipperPath->text());

	if(file != "") {
		ui->Global_StreamRipperPath->setText(file);
	}

	SaveGlobalPreferences();

}


// The fields for details for some preferences are dis-/enabled according to the checking-status
void MainWindow::DisEnablePrefDetailsFields() {

	if(ui->Pref_D->isChecked()) {
		ui->Pref_D_Details->setEnabled(true);
	} else {
		ui->Pref_D_Details->setEnabled(false);
	}

	if(ui->Pref_a->isChecked()) {
		ui->Pref_a_Details->setEnabled(true);
	} else {
		ui->Pref_a_Details->setEnabled(false);
	}

	if(ui->Pref_o->isChecked()) {
		ui->Pref_o_Details->setEnabled(true);
	} else {
		ui->Pref_o_Details->setEnabled(false);
	}

	if(ui->Pref_l->isChecked()) {
		ui->Pref_l_Details->setEnabled(true);
	} else {
		ui->Pref_l_Details->setEnabled(false);
	}

	if(ui->Pref_M->isChecked()) {
		ui->Pref_M_Details->setEnabled(true);
	} else {
		ui->Pref_M_Details->setEnabled(false);
	}

	if(ui->Pref_q->isChecked()) {
		ui->Pref_q_Details->setEnabled(true);
	} else {
		ui->Pref_q_Details->setEnabled(false);
	}

	if(ui->Pref_u->isChecked()) {
		ui->Pref_u_Details->setEnabled(true);
	} else {
		ui->Pref_u_Details->setEnabled(false);
	}


}


// The streamripper preferences "-D" and "-a" have to have a specific pattern -> check
void MainWindow::RegExpPrefFields() {

	if(ui->Pref_D->isChecked()) {
		QRegExpValidator *valid_pref_D = new QRegExpValidator(QRegExp("^([%][ATDdSq]){1}(/[%][ATDdSq]){0,}$"), this);
		QString text = ui->Pref_D_Details->text();
		int pos = 0;
		if(valid_pref_D->validate(text,pos) < 2) {
			 QPalette palette;
			 palette.setColor(ui->Pref_D_Details->backgroundRole(),QColor(255,99,99));
			 ui->Pref_D_Details->setPalette(palette);
		} else {
			QPalette palette;
			palette.setColor(ui->Pref_D_Details->backgroundRole(),QColor(123,255,117));
			ui->Pref_D_Details->setPalette(palette);
		}
	} else {
		QPalette palette;
		palette.setColor(ui->Pref_D_Details->backgroundRole(),QColor(255,255,255));
		ui->Pref_D_Details->setPalette(palette);
	}


	if(ui->Pref_a->isChecked()) {
		QRegExpValidator *valid_pref_a = new QRegExpValidator(QRegExp("^([%][ATDdSq]){0,1}(/[%][ATDdSq]){0,}$"), this);
		QString text = ui->Pref_a_Details->text();
		int pos = 0;
		if(valid_pref_a->validate(text,pos) < 2) {
			 QPalette palette;
			 palette.setColor(ui->Pref_a_Details->backgroundRole(),QColor(255,99,99));
			 ui->Pref_a_Details->setPalette(palette);
		} else {
			QPalette palette;
			palette.setColor(ui->Pref_a_Details->backgroundRole(),QColor(123,255,117));
			ui->Pref_a_Details->setPalette(palette);
		}
	} else {
		QPalette palette;
		palette.setColor(ui->Pref_a_Details->backgroundRole(),QColor(255,255,255));
		ui->Pref_a_Details->setPalette(palette);
	}

}


// Add a whole new station to the List (default values
void MainWindow::AddNewStation() {

	state(1,"Adding New Station To List.");

	// Save the sorting and remove sorting
	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();
	ui->list->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);

	QSqlQuery query;

	QStringList codes;

	query.exec("SELECT code FROM details");
	while(query.next()) {
		codes << query.value(query.record().indexOf("code")).toString();
	}

	SQLERR(stations,query,"AddNewStation 1");

	query.clear();

	QStringList possibleCharacters;
	possibleCharacters << "a" << "b" << "c" << "d" << "e" << "f" << "g" << "h" << "i" << "j" << "k" << "l" << "m" << "n" << "o" << "p" << "q" << "r" << "s" << "t" << "u" << "v" << "w" << "x" << "y" << "z"
			<< "1" << "2" << "3" << "4" << "5" << "6" << "7" << "7" << "8" << "9" << "0"
			<< "#" << "+" << "-" << "!" << "=" << "?" << "&" << "$";

	QString string = "";

	int new_string = 0;

	while(new_string == 0) {

		for(int i = 0; i < 10; ++i) {

			int random = qrand() % possibleCharacters.size();

			string += possibleCharacters.at(random);

		}

		query.exec("SELECT COUNT(code) FROM details WHERE code LIKE '" + string + "'");
		query.next();
		if(query.value(0).toString() == "0") {
			new_string = 1;
		}
		query.clear();

	}

	// Here the code of last added stream is saved (necessary for adding a stream from a directory)
	codeOfLastAdded = string;

	QString new_name = tr("[New Stream]");

	query.exec("INSERT INTO details (code,D,s,r,a_text,a_checked,o,c,l,M,q,u,i,wid3v1,woid3v2,name,url,location,server,bitrate,metainterval,swapartisttitle,playback)"
		   "VALUES('" + string + "','%A/%T','0','1','','0','2','0','0','0','0','','0','0','0','" + new_name + "','http://','" + ui->Global_DefaultLocation->text() + "','---','---','---','0','normal')");

	values[string]["localhost"] = "---";
	values[string]["recording"] = "0";
	values[string]["playing"] = "0";
	values[string]["playback"] = "intelligent";
	values[string]["stopaftersong"] = "0";
	values[string]["startrecording"] = "0";
	values[string]["stoprecording"] = "0";
	values[string]["timeout"] = "0";

	SQLERR(stations,query,"AddNewStation 2");

	query.clear();

	int row = ui->list->rowCount();
	ui->list->setRowCount(row+1);

	// Enter new default data
	QTableWidgetItem *item = new QTableWidgetItem("");
	item->setIcon(QIcon(":/images/StreamStopRecording.png"));
	item->setStatusTip(new_name);
	ui->list->setItem(row,0,item);

	item = new QTableWidgetItem(status_notconnected);
	item->setStatusTip(new_name);
	ui->list->setItem(row,1,item);

	item = new QTableWidgetItem(new_name);
	item->setStatusTip(new_name);
	ui->list->setItem(row,2,item);

	item = new QTableWidgetItem("[" + id3_artist + "]");
	item->setStatusTip(new_name);
	ui->list->setItem(row,3,item);

	item = new QTableWidgetItem("[" + id3_title + "]");
	item->setStatusTip(new_name);
	ui->list->setItem(row,4,item);

	item = new QTableWidgetItem("---");
	item->setStatusTip(new_name);
	ui->list->setItem(row,5,item);

	item = new QTableWidgetItem(string);
	item->setStatusTip(new_name);
	ui->list->setItem(row,6,item);

	// And select the new row
	ui->list->selectRow(row);
	SelectEntry(row,1);

	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else if(order_ascdesc == 1) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}

}


// Delete a station from the list
void MainWindow::DeleteStation() {

	state(1,"Deleting Station");

	QString code = ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text();

	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();
	ui->list->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);

	int row = ui->list->selectedItems().at(0)->row();

	QSqlQuery query;

	query.exec("SELECT COUNT(code) FROM details");
	query.next();

	// If there are more than one stream in the list
	if(query.value(0).toString() != "1") {

		query.exec("DELETE FROM details WHERE code LIKE '" + code + "'");
		query.clear();

		values.remove(code);

		ui->list->removeRow(row);


	// If the last stream shall be deleted -> reset instead of delete
	} else {

		QString new_name = tr("[New Stream]");

		query.exec("UPDATE details SET D='%A/%T',s='0',r='1',a_text='',a_checked='0',o='2',c='0',l='0',M='0',q='0',u='',i='0',wid3v1='0',woid3v2='0',name='" + new_name + "',url='http://',location='" + QDir::homePath() + "/',server='---',bitrate='---',metainterval='---' WHERE code LIKE '" + code + "'");

		values[code]["localhost"] = "---";
		values[code]["recording"] = "0";
		values[code]["playing"] = "0";
		values[code]["playback"] = "normal";
		values[code]["stopaftersong"] = "0";
		values[code]["startrecording"] = "0";
		values[code]["stoprecording"] = "0";
		values[code]["timeout"] = "0";

		ui->list->item(0,1)->setText(status_notconnected);
		ui->list->item(0,2)->setText(new_name);
		ui->list->item(0,3)->setText("[" + id3_artist + "]");
		ui->list->item(0,4)->setText("[" + id3_title + "]");
		ui->list->item(0,5)->setText("---");

		query.clear();

	}

	SQLERR(stations,query,"DeleteStation");

	query.clear();

	if(row == ui->list->rowCount()) {
		ui->list->selectRow(row-1);
		SelectEntry(row-1,1);
	} else {
		ui->list->selectRow(row);
		SelectEntry(row,1);
	}


	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else if(order_ascdesc == 1) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}


}


// The recording of the selected stream shall be started/stopped
void MainWindow::Sel_StartRecording() {

	QString code = ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text();

	bool ok;
	int recording = values[code]["recording"].toInt(&ok,10);

	if(recording == 0) {
		values[code]["startrecording"] = "1";
	} else {
		values[code]["stoprecording"] = "1";
	}


	StartStopRecording();

}


// The recording of the selected stream shall be stopped after current song
void MainWindow::Sel_StopAfterSong() {

	int row = ui->list->selectedItems().at(0)->row();

	QString code = ui->list->item(row,6)->text();

	if(values[code]["stopaftersong"] == "1") {

		values[code]["stopaftersong"] = "0";
		ui->list->item(row,1)->setIcon(QIcon(""));

	} else {

		if(values[code]["recording"] == "1" && values[code]["localhost"] != "---" && values[code]["localhost"] != "") {

			values[code]["stopaftersong"] = "1";

			ui->list->item(row,1)->setIcon(QIcon(":/images/StreamStopRecordingAfterSong.png"));

		}

	}

}


// The Recording of all streams shall be started
void MainWindow::All_StartRecording() {

	for(int i = 0; i < ui->list->rowCount(); ++i) {

		QString code = ui->list->item(i,6)->text();

		if(values[code]["recording"] == "0") {
			values[code]["startrecording"] = "1";
		}

	}

	StartStopRecording();

}


// The recording of all streams shall be stopped
void MainWindow::All_StopRecording() {

	QMap<QString, QMap<QString,QString> >::iterator i = values.begin();
	while(i != values.end()) {
		if(i.value()["recording"] == "1") {
			values[i.key()]["stoprecording"] = "1";
		}
		++i;
	}

	StartStopRecording();

}


// The recording of all streams shall be stopped after current song
void MainWindow::All_StopAfterSong() {

	for(int i = 0; i < ui->list->rowCount(); ++i) {

		QString code = ui->list->item(i,6)->text();

		if(values[code]["stopaftersong"] == "1") {

			values[code]["stopaftersong"] = "0";
			ui->list->item(i,1)->setIcon(QIcon(""));

		} else {

			if(values[code]["recording"] == "1" && values[code]["localhost"] != "---" && values[code]["localhost"] != "") {

				values[code]["stopaftersong"] = "1";

				ui->list->item(i,1)->setIcon(QIcon(":/images/StreamStopRecordingAfterSong.png"));

			}

		}

	}

}


// Start/Stop recording of marked stations
void MainWindow::StartStopRecording() {

	QSqlQuery query;

	// All the stations, whose recording shall be started
	QStringList codes;
	QMap<QString, QMap<QString,QString> >::iterator i = values.begin();
	while (i != values.end()) {
		if(i.value()["startrecording"] == "1") {
			codes << i.key();
		}
		++i;
	}

	// When a stream was played and a recording was started, the playing is restarted after a succesful start
	int startplayingagain = -1;

	// start all of them
	for(int g = 0; g < codes.size(); ++g) {

		QString code = codes.at(g);

		// Create the process
		stations_processes[code] = new QProcess;

		// Get all the preferences
		query.exec("SELECT url FROM details WHERE code LIKE '" + code + "'");
		query.next();

		for(int i = 0; i < ui->list->rowCount(); ++i) {

			if(ui->list->item(i,6)->text() == code) {

				// mms:// streams cannot be recorded only played
				if(query.value(query.record().indexOf("url")).toString().startsWith("mms")) {

					state(4,"Sorry, can't record mms:// streams yet");

					QMessageBox::information(this,tr("Can't rip it!"),tr("Sorry, Windows Media Streams (mms://) cannot be recorded yet, only played (normal playback only)!!"));

					stations_processes[code]->kill();

					values[code]["startrecording"] = "0";

				} else {

					// Start recording

					state(1,"Started Recording of: " + ui->list->item(i,2)->text());

					ui->list->item(i,1)->setTextColor(Qt::red);
					ui->list->item(i,2)->setTextColor(Qt::red);
					ui->list->item(i,3)->setTextColor(Qt::red);
					ui->list->item(i,4)->setTextColor(Qt::red);
					ui->list->item(i,5)->setTextColor(Qt::red);

					ui->list->item(i,0)->setIcon(QIcon(":/images/StreamStartRecording.png"));

					// The first argument is the URL
					QStringList arguments;
					arguments << query.value(query.record().indexOf("url")).toString();

					// The rest is created by the function
					arguments.append(CreateStreamripperPrefString(code));

					// Start the process
					stations_processes[code] = new QProcess();
					stations_processes[code]->start(ui->Global_StreamRipperPath->text(), arguments);

					// And connect process with function
					connect(stations_processes[code], SIGNAL(readyRead()), this, SLOT(GetStreamripperOutput()));

					values[code]["recording"] = "1";
					values[code]["startrecording"] = "0";

					ui->list->item(i,1)->setText(status_connecting);

					// If stream is selected -> disable some stuff
					if(i == ui->list->selectedItems().at(0)->row()) {

						ui->Preferences->setEnabled(false);
						ui->Details_IntelligentPlayback->setEnabled(false);
						ui->Details_URLEdit->setEnabled(false);
						ui->Details_LocationEdit->setEnabled(false);
						ui->Details_LocationChoose->setEnabled(false);

						sel_record->setIcon(QIcon(":/images/StreamStopRecording.png"));
						ui->Details_Record->setIcon(QIcon(":/images/StreamStopRecording.png"));

					}

				}

				if(code == played) {

					startplayingagain = i;

				}

			}

		}

	}

	SQLERR(stations,query,"StartStopRecording 1");

	query.clear();


	codes.clear();

	// All the streams, whose recording shall be stopped
	QMap<QString, QMap<QString,QString> >::iterator j = values.begin();
	while (j != values.end()) {
		if(j.value()["stoprecording"] == "1") {
			codes << j.key();
		}
		++j;
	}

	for(int i = 0; i < codes.size(); ++i) {

		QString code = codes.at(i);

		stations_processes[code]->kill();

		int row = -1;
		for(int i = 0; i < ui->list->rowCount(); ++i) {
			if(ui->list->item(i,6)->text() == code) {
				row = i;
			}
		}

		if(row != -1) {

			// Stop possible playing
			if(played == code) {

				int selectedbefore = ui->list->selectedItems().at(0)->row();

				ui->list->selectRow(row);

				StartStopPlaying();

				ui->list->selectRow(selectedbefore);

			}

			state(1,"Stopped recording of: " + ui->list->item(row,2)->text());

			ui->list->item(row,0)->setIcon(QIcon(":/images/StreamStopRecording.png"));

			// Reset list view

			ui->list->item(row,1)->setTextColor(Qt::black);
			ui->list->item(row,2)->setTextColor(Qt::black);
			ui->list->item(row,3)->setTextColor(Qt::black);
			ui->list->item(row,4)->setTextColor(Qt::black);
			ui->list->item(row,5)->setTextColor(Qt::black);

			ui->list->item(row,1)->setIcon(QIcon(""));

			ui->list->item(row,1)->setText(status_notconnected);
			ui->list->item(row,3)->setText("[" + id3_artist + "]");
			ui->list->item(row,4)->setText("[" + id3_title + "]");
			ui->list->item(row,5)->setText("---");
			ui->Details_ArtistTitle->setText("<p align=\"center\"><b>" + id3_artist + "</b> - " + id3_title + "</p>");

			QString server = ui->Details_ServerName->text();
			QString bitrate = ui->Details_Bitrate->text();
			QString metainterval = ui->Details_MetaInterval->text();

			ui->Details_ServerName->setText(server);
			ui->Details_Bitrate->setText(bitrate);
			ui->Details_MetaInterval->setText(metainterval);

			values[code]["recording"] = "0";
			values[code]["stoprecording"] = "0";

			// If selected, enable some stuff
			if(row == ui->list->selectedItems().at(0)->row()) {

				ui->Preferences->setEnabled(true);
				ui->Details_IntelligentPlayback->setEnabled(true);
				ui->Details_URLEdit->setEnabled(true);
				ui->Details_LocationEdit->setEnabled(true);
				ui->Details_LocationChoose->setEnabled(true);

				sel_record->setIcon(QIcon(":/images/StreamStartRecording.png"));
				ui->Details_Record->setIcon(QIcon(":/images/StreamStartRecording.png"));

			}

			query.exec("UPDATE details SET server='" + server + "',bitrate='" + bitrate + "',metainterval='" + metainterval + "' WHERE code LIKE '" + code + "'");

			SQLERR(stations,query,"StartStopRecording 1");

			query.clear();

		}
	}

	// If playing shall be restarted
	if(startplayingagain > -1) {

		int selectedbefore = ui->list->selectedItems().at(0)->row();

		ui->list->selectRow(startplayingagain);

		torestart = startplayingagain;
		startplayingagain = -1;

		StartStopPlaying();

		ui->list->selectRow(selectedbefore);

	}

}


// Start/stop playing of a stream
void MainWindow::StartStopPlaying() {

	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();

	// If a stream is already played -> stop
	if(played != "") {

		ui->list->horizontalHeader()->setSortIndicator(-1,Qt::AscendingOrder);

		int row = 0;

		int i = 0;
		while(i < ui->list->rowCount()) {
			if(ui->list->item(i,6)->text() == played) {
				row = i;
				break;
			}
			++i;
		}

		std::cout << "[INFO] Stopped Playing of: " << ui->list->item(row,2)->text().toStdString() << std::endl;

		// If not recorded -> complete reset
		if(values[played]["recording"] == "0") {

			ui->list->item(row,1)->setTextColor(Qt::black);
			ui->list->item(row,2)->setTextColor(Qt::black);
			ui->list->item(row,3)->setTextColor(Qt::black);
			ui->list->item(row,4)->setTextColor(Qt::black);
			ui->list->item(row,5)->setTextColor(Qt::black);

			ui->list->item(row,0)->setIcon(QIcon(":/images/StreamStopRecording.png"));
			ui->list->item(row,3)->setText("[" + id3_artist + "]");
			ui->list->item(row,4)->setText("[" + id3_title + "]");
			ui->list->item(row,5)->setText("---");

			if(values[played]["playback"] == "intelligent") {
				streamripperforplaying->terminate();
				while(streamripperforplaying->waitForFinished(2000)) {
				}
				playinglocalhost = "";
				playingstarted = 0;
			}

			if(save_song->hasFocus()) {
				ui->Details_Play->setFocus();
			}

			
			if(ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text() == played) {
				ui->Details_ArtistTitle->setText("<p align=\"center\"><b>" + id3_artist + "</b> - " + id3_title + "</p>");
				save_song->setEnabled(false);
				save_song->setChecked(false);
				sel_play->setIcon(QIcon(":/images/StreamPlay.png"));
				ui->Details_Play->setIcon(QIcon(":/images/StreamPlay.png"));
			}

			ui->list->item(row,1)->setText(status_notconnected);

		} else {

			ui->list->item(row,2)->setIcon(QIcon(""));
		}

		mplayer->terminate();
		while(mplayer->waitForFinished(2000)) {
		}

		played_values.clear();

		values[played]["playing"] = "0";

		this->setWindowTitle(version);

	}

	QString code = ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text();

	// If the playing shall only be stopped
	if(played == code) {

		QString name = ui->list->item(ui->list->findItems(played,Qt::MatchCaseSensitive).at(0)->row(),2)->text();

		showMessage(1,"Playing Stopped","Stopped Playing of '" + name + "'.");

		played = "";

		ui->Details_NormalPlayback->setEnabled(true);
		ui->Details_IntelligentPlayback->setEnabled(true);
		ui->Details_PlaybackLabel->setEnabled(true);

		ui->Lyric_Artist->setEnabled(false);
		ui->Lyric_Artist_label->setEnabled(false);
		ui->Lyric_Display->setEnabled(false);
		ui->Lyric_Fetch->setEnabled(false);
		ui->Lyric_Reset->setEnabled(false);
		ui->Lyric_Title->setEnabled(false);
		ui->Lyric_Title_label->setEnabled(false);

		UpdateLyricsInterfaces("---",tr("<p align=\"center\"><br><i>[No Stream Playing]</i></p>"),0);

		ui->Lyric_Artist->setText("");
		ui->Lyric_Title->setText("");

	// Start playing of selected stream
	} else {

		ui->Details_NormalPlayback->setEnabled(false);
		ui->Details_IntelligentPlayback->setEnabled(false);
		ui->Details_PlaybackLabel->setEnabled(false);

		int sel = ui->list->selectedItems().at(0)->row();

		sel_play->setIcon(QIcon(":/images/StreamStop.png"));
		ui->Details_Play->setIcon(QIcon(":/images/StreamStop.png"));

		state(1,"Started Plaing of: " + ui->list->item(sel,2)->text());
		showMessage(1,"Playing Started","Started Playing of '" + ui->list->item(sel,2)->text() + "'");

		if(values[code]["recording"] == "0") {

			ui->list->item(sel,1)->setTextColor(Qt::blue);
			ui->list->item(sel,2)->setTextColor(Qt::blue);
			ui->list->item(sel,3)->setTextColor(Qt::blue);
			ui->list->item(sel,4)->setTextColor(Qt::blue);
			ui->list->item(sel,5)->setTextColor(Qt::blue);

			ui->list->item(sel,0)->setIcon(QIcon(":/images/StreamPlay.png"));
			ui->list->item(sel,1)->setText(status_playing);

		} else {

			ui->list->item(sel,2)->setIcon(QIcon(":/images/StreamPlay.png"));

		}

		this->setWindowTitle(tr("Connecting... -- ") + version);

		if(values[code]["recording"] == "1") {
			if(values[code]["localhost"] != "" && values[code]["localhost"] != "---") {
				QStringList mplayerArguments;
				mplayerArguments << "http://localhost:" + values[code]["localhost"] << "-cache" << "32";
				mplayerArguments << "-msglevel" << "all=4" << "-idle" << "-slave";
				mplayer->start(ui->Global_MPlayerPath->text(),mplayerArguments);
			} else {
				torestart = sel;
			}
		} else {

			if(values[code]["playback"] == "intelligent") {

				QDir dir;
				DeleteDirectoryRecursive(QDir::tempPath() + "/rripper/");
				dir.mkdir(QDir::tempPath() + "/rripper/");

				QStringList arguments;
				arguments << ui->Details_URLEdit->text() << "-r" << "-s" << "-d" << QDir::tempPath() + "/rripper/" << "-k" << "0";

				streamripperforplaying->start(ui->Global_StreamRipperPath->text(),arguments);


				playingstarted = 0;

			} else {

				QStringList arguments;
				arguments << ui->Details_URLEdit->text() << "-msglevel" << "all=4"  <<"-idle" << "-slave";
				mplayer->start(ui->Global_MPlayerPath->text(),arguments);


			}

		}

		played = code;

		values[code]["playing"] = "1";

		// Possibly fetch lyrics
		if(ui->Lyric_AutoFetch->isChecked()) {

			ui->Lyric_Artist->setEnabled(false);
			ui->Lyric_Artist_label->setEnabled(false);
			ui->Lyric_Display->setEnabled(false);
			ui->Lyric_Fetch->setEnabled(false);
			ui->Lyric_Reset->setEnabled(false);
			ui->Lyric_Title->setEnabled(false);
			ui->Lyric_Title_label->setEnabled(false);

			UpdateLyricsInterfaces("---",tr("<p align=\"center\"><br><i>[Fetching]</i></p>"),0);

		} else {

			ui->Lyric_Artist->setEnabled(true);
			ui->Lyric_Artist_label->setEnabled(true);
			ui->Lyric_Display->setEnabled(false);
			ui->Lyric_Fetch->setEnabled(true);
			ui->Lyric_Reset->setEnabled(true);
			ui->Lyric_Title->setEnabled(true);
			ui->Lyric_Title_label->setEnabled(true);

			UpdateLyricsInterfaces("---",tr("<p align=\"center\"><br><i>[Auto-fetch disabled]</i></p>"),0);

		}

	}

	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}

}


// Get the streamripper output
void MainWindow::GetStreamripperOutput() {

	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();

	ui->list->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);

	QMap<QString, QProcess*>::const_iterator i = stations_processes.constBegin();

	QSqlQuery query;

	// Proof any streams for output
	while (i != stations_processes.constEnd()) {

		QString code = i.key();

		int row = -1;
		for(int t = 0; t < ui->list->rowCount(); ++t) {
			if(ui->list->item(t,6)->text() == code) {
				row = t;
			}
		}


		QString output = i.value()->readLine(0);

		output = output.trimmed();

		// If something went wrong...
		if(output.startsWith("shutting down") || output.startsWith("bye")) {
			query.exec("SELECT c FROM details WHERE code LIKE '" + code + "'");
			query.next();
			if(query.value(query.record().indexOf("c")).toString() == "1") {
				std::cout << "Connection to stream dropped. I don't try to reconnect..." << std::endl << std::endl << "If this shouldn't have happened, disable the streamripper option 'c' and try again. If it happens again, please report it as a bug to 'rripper91@yahoo.com'" << std::endl << std::endl;
			} else {
				QMessageBox::critical(this,tr("Error"),tr("An error occured! Streamripper will be stopped.\n\n\nPlease make first sure you have a working connection to the internet and that the URL is right.\n\nIf this is the case, then please report this as a bug to 'rripper91@yahoo.com'.\n(In the report, mention your version of RRipper and the URL you tried to connect with)"));
			}
			SQLERR(stations,query,"GetStreamripperOutput 1");
			query.clear();

			query.exec("UPDATE details SET stoprecording='1' WHERE code LIKE '" + code + "'");
			SQLERR(stations,query,"GetStreamripperOutput 2");
			query.clear();
		} else {

			if(output.trimmed() != "" && row != -1) {

				// Output: "Connecting..."
				if(output.startsWith("Connecting")) {

					ui->list->item(row,1)->setText(status_connecting);

				}

				// Output: "stream: xxxxxxxxx"
				if(output.startsWith("stream")) {

					ui->list->item(row,1)->setText(status_receivingdetails);

					QString ReceivedName = output.split(": ").at(1);

					// If received and saved name don't match
					if(ReceivedName.trimmed() != ui->list->item(row,2)->text().trimmed()) {

						int msg = QMessageBox::Cancel;

						if(ui->Global_Matching->currentIndex() == 0) {

							msg = QMessageBox::Cancel;

						} else if(ui->Global_Matching->currentIndex() == 1) {

							msg = QMessageBox::Ok;

						} else if(ui->Global_Matching->currentIndex() == 2) {

							msg = QMessageBox::information(this,
									tr("Received Streamname"),
									tr("The received streamname differs from the saved one.\n\nSaved: %1\n Received: %2\n\nDo you want to adapt the new name?").arg(ui->list->item(row,2)->text()).arg(ReceivedName),
									QMessageBox::Ok | QMessageBox::Default, QMessageBox::Cancel);

						}

						if(msg == QMessageBox::Ok) {

							ui->list->item(row,2)->setText(ReceivedName);
							ui->Details_StreamNameEdit->setText(ReceivedName);
							ui->Details_StreamName->setText("<p align=\"center\" style=\"font-size: 12pt\"><b>" + ReceivedName + "</b></p>");

							query.exec("UPDATE details SET name='" + ReceivedName + "' WHERE code LIKE '" + code + "'");
							SQLERR(stations,query,"GetStreamripperOutput 3");
							query.clear();

						}

					}

				}

				// Output: "servername: xxxxxx"
				if(output.startsWith("server name")) {

					ui->list->item(row,1)->setText(status_receivingdetails);

					QString servername = output.split(": ").at(1);

					if(servername != ui->Details_ServerName->text()) {
						ui->Details_ServerName->setText(servername);
						query.exec("UPDATE details SET server='" + servername + "' WHERE code LIKE '" + code + "'");
						SQLERR(stations,query,"GetStreamripperOutput 4");
						query.clear();
					}

				}

				// Output: "declared bitrate: xxx"
				if(output.startsWith("declared bitrate")) {

					ui->list->item(row,1)->setText(status_receivingdetails);

					QString bitrate = output.split(": ").at(1) + " kbit/s";

					if(bitrate != ui->Details_Bitrate->text()) {
						ui->Details_Bitrate->setText(bitrate);
						query.exec("UPDATE details SET bitrate='" + bitrate + "' WHERE code LIKE '" + code + "'");
						SQLERR(stations,query,"GetStreamripperOutput 5");
						query.clear();
					}

				}

				// Output: "meta interval: xxxxxx"
				if(output.startsWith("meta interval")) {

					ui->list->item(row,1)->setText(status_receivingdetails);

					QString metainterval = output.split(": ").at(1);

					if(metainterval != ui->Details_MetaInterval->text()) {
						ui->Details_MetaInterval->setText(metainterval);
						query.exec("UPDATE details SET metainterval='" + metainterval + "' WHERE code LIKE '" + code + "'");
						SQLERR(stations,query,"GetStreamripperOutput 6");
						query.clear();
					}

				}

				// Output "relay port: xxxx"
				if(output.startsWith("relay port")) {

					ui->list->item(row,1)->setText(status_receivingdetails);

					QString localhost = output.split(": ").at(1);

					values[code]["localhost"] = localhost;

				}

				// Output: "[getting track name...]"
				if(output.startsWith("[getting")) {

					ui->list->item(row,1)->setText(status_gettingtrackname);

				}

				// Output: "[buffering...]"
				if(output.startsWith("[buffering")) {

					ui->list->item(row,1)->setText(status_buffering);

				}

				// Output: "[skipping...    ] xxxxxxx - xxxxxx  [  xxx]"
				if(output.startsWith("[skipping")) {

					ui->list->item(row,1)->setText(status_skipping);

				}

				// Output: "[ripping...    ] xxxxxxx - xxxxxx  [  xxx]"
				if(output.startsWith("[ripping")) {

					ui->list->item(row,1)->setText(status_ripping);

				}

				// In these three cases there is id3 and filesize information
				if(output.startsWith("[buffering") || output.startsWith("[skipping") || output.startsWith("[ripping")) {

					// Am I ripping or skipping?
					QString IAmDoing;
					if(output.startsWith("[ripping")) {
						IAmDoing = "RIPPING";
					} else {
						IAmDoing = "SKIPPING";
					}

					// When this stream shall be restarted, then do so
					if(torestart == row) {
						int selectedbefore = ui->list->selectedItems().at(0)->row();
						ui->list->selectRow(row);
						StartStopPlaying();
						ui->list->selectRow(selectedbefore);
						torestart = -1;
					}

					output = output.remove("[skipping...   ]").trimmed();
					output = output.remove("[buffering...   ]").trimmed();
					output = output.remove("[ripping...    ]").trimmed();

					// Filzer id3 and filesize information out
					QString id3 = output.split("[").at(0).trimmed();
					QString Size = output.split("[").at(1).trimmed().split("]").at(0).trimmed();

					QString title;
					QString artist;

					if(id3 == "") {

						title = tr("[No ID3 available]");
						artist = tr("[No ID3 available]");

					} else {

						// These are the id3 from the time before
						QString artist_old = ui->list->item(row,3)->text();
						QString title_old = ui->list->item(row,4)->text();

						QString split_string = "";

						int split = 0;
						int pos = 0;

						QRegExpValidator *id3_name = new QRegExpValidator(QRegExp("^([0-9]){0,3} (\\S|\\s)*$"),this);

						// Split id3 into artist and title
						if(id3.contains(" - - ")) {

							split_string = "- -";
							split = 1;

						} else if(id3.contains(" - ")) {

							split_string = " - ";
							split = 1;

						} else if(id3.contains("--")) {

							split_string = "--";
							split = 1;

						} else if(id3_name->validate(id3,pos)) {

							QStringList split = id3.split(" ");
							split.removeFirst();
							id3 = split.join(" ");

						} else if(id3.contains("-")) {

							split_string = "-";
							split = 1;

						}

						if(split == 1) {

							artist = id3.split(split_string).at(0);
							title = id3.split(split_string).at(1);

						} else {

							artist = "";
							title = id3;

						}

						// Some stations have the spaces replaces with '_'
						artist = artist.replace("_"," ");
						title = title.replace("_"," ");

						// Remove whitespaces left and right
						artist = artist.trimmed();
						title = title.trimmed();

						// Shall I swap the displaying of artist/title
						query.exec("SELECT swapartisttitle FROM details WHERE code LIKE '" + code + "'");
						query.next();

						if(query.value(query.record().indexOf("swapartisttitle")).toString() == "1") {
							QString temp = artist;
							artist = title;
							title = temp;
						}

						SQLERR(stations,query,"GetStreamripperOutput 7");

						query.clear();

						// If the id3 has changed
						if(artist_old != artist || title_old != title) {

							if(values[code]["stopaftersong"] == "1") {
								values[code]["stoprecording"] = "1";
								values[code]["stopaftersong"] = "0";
								StartStopRecording();
							} else {

								state(1,IAmDoing + ": " + artist.toUpper() + " - " + title.toUpper());

								title = title.toLower();
								QStringList title_parts = title.split(" ");
								QString title_edited = "";
								if(title.size() > 0) {
									for(int i = 0; i < title_parts.length(); ++i) {
										QString title_current = title_parts.at(i).toLower();
										QString title_first = title_current.at(0).toUpper();
										title_current = title_current.remove(0,1);
										title_edited += title_first + title_current + " ";
									}
									title = title_edited.trimmed();
								}


								artist = artist.toLower();
								QStringList artist_parts = artist.split(" ");
								QString artist_edited = "";
								if(artist.size() > 0) {
									for(int i = 0; i < artist_parts.length(); ++i) {
										QString artist_current = artist_parts.at(i).toLower();
										QString artist_first = artist_current.at(0).toUpper();
										artist_current = artist_current.remove(0,1);
										artist_edited += artist_first + artist_current + " ";
									}
									artist = artist_edited.trimmed();
								}

								ui->Lyric_Artist->setText(artist);
								ui->Lyric_Title->setText(title);

								ui->list->item(row,3)->setText(artist);
								ui->list->item(row,4)->setText(title);

								if(ui->Lyric_AutoFetch->isChecked()) {
									RequestLyrics();
								}

							}

						}

						// If stream is selected
						if(row == ui->list->selectedItems().at(0)->row()) {
							QString text;
							text = "<p align=\"center\">";
							if(artist != "") {
								text += "<b>" + artist + "</b> - ";
							}
							text += title + "</p>";
							ui->Details_ArtistTitle->setText(text);
						}

					}

					// Get the size
					if(Size.endsWith("kb") || Size.endsWith("M")) {

						if(Size.endsWith("kb")) {
							Size = Size.remove(Size.size() - 2, Size.size()) + " KB";
						} else if(Size.endsWith("M")) {
							Size = Size.remove(Size.size() - 1, Size.size()) + " MB";
						}

						ui->list->item(row,5)->setText(Size);

					}

				}

			}
		}
		++i;
	}

	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}

	StartStopRecording();

}


// Get the mplayer output (NEED TO BE REVISED)
void MainWindow::GetMplayerOutput() {

	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();

	ui->list->horizontalHeader()->setSortIndicator(-1,Qt::AscendingOrder);

	QString code_sel = ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text();

	QString output = mplayer->readAll();

	output = output.remove("\n");

	if(output.contains("No stream found to handle url")) {

		QSqlQuery query;
		query.exec("SELECT url FROM details WHERE code LIKE '" + played + "'");
		query.next();

		QMessageBox::information(this,tr("Connection error"),tr("RRipper wasn't able to connect to the URL '%1'!\n\nThe service is either currently unavailable or the URL is not correct.").arg(query.value(query.record().indexOf("url")).toString()));

		SQLERR(stations,query,"GetMPlayerOutput 1");

		query.clear();

		StartStopPlaying();

		if(played != ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text()) {
			StartStopPlaying();
		}


	} else if(output.contains("ICY Info:") && values[played]["recording"] == "0") {

		// Remove the stuff at the beginning ("ICY Info: StreamTitle='")
		QStringList temp = output.split("ICY Info: StreamTitle='");
		if(temp.size() > 1) {
			output = temp.at(1);
		}
		output = output.split("';StreamUrl=").first();
		output = output.split("';").first();

		output = output.replace("_"," ");

		QString artist = "";
		QString title = "";

		temp = output.split(" - - ");
		if(temp.size() == 2) {
			artist = temp.at(0);
			title = temp.at(1);
		} else if(temp.size() == 1) {
			temp = temp.at(0).split("--");
			if(temp.size() > 1) {
				artist = temp.first();
				title = temp.at(1);
			} else {
				temp = temp.at(0).split("- -");
				if(temp.size() == 1) {
					temp = temp.at(0).split("-");
					if(temp.size() == 1) {
						title = temp.first();
					} else {
						artist = temp.first();
						title = temp.at(1);
					}
				} else {
					artist = temp.at(0),
					title = temp.at(1);
				}
			}
		}

		artist = artist.replace("_"," ");
		title = title.replace("_"," ");

		artist = artist.trimmed();
		title = title.trimmed();

		QSqlQuery query;

		query.exec("SELECT swapartisttitle FROM details WHERE code LIKE '" + played + "'");
		query.next();

		if(query.value(query.record().indexOf("swapartisttitle")).toString() == "1") {
			QString temp = artist;
			artist = title;
			title = temp;
		}

		SQLERR(stations,query,"GetMPlayerOutput 2");

		query.clear();

		if(title == "" && artist == "") {
			std::cout << "Error in handling meta information: " << output.toStdString() << std::endl << "Please write a bug report to 'rripper91@yahoo.com'." << std::endl << std::endl;
		} else if(title == "" && artist != "") {
			title = artist;
			artist = "";
		}

		if(values[played]["playback"] == "normal") {

			QString displayinfo = "<p align=\"center\">";
			if(artist != "") {
				displayinfo += "<b>" + artist + "</b> - ";
			}
			displayinfo += title + "</p>";

			ui->Details_ArtistTitle->setText(displayinfo);

			int mplayer_row = 0;
			QList<QTableWidgetItem*> list = ui->list->findItems(played,Qt::MatchExactly);
			mplayer_row = list.at(0)->row();

			QString artist_old = ui->list->item(mplayer_row,3)->text();
			QString title_old = ui->list->item(mplayer_row,4)->text();

			ui->list->item(mplayer_row,3)->setText(artist);
			ui->list->item(mplayer_row,4)->setText(title);

			if((artist_old != artist || title_old != title)) {

				state(1,"PLAYING: " + artist.toUpper() + " - " + title.toUpper());

				title = title.toLower();
				QStringList title_parts = title.split(" ");
				QString title_edited = "";
				for(int i = 0; i < title_parts.length(); ++i) {
					QString title_current = title_parts.at(i).toLower();
					QString title_first = title_current.at(0).toUpper();
					title_current = title_current.remove(0,1);
					title_edited += title_first + title_current + " ";
				}
				title = title_edited.trimmed();


				artist = artist.toLower();
				QStringList artist_parts = artist.split(" ");
				QString artist_edited = "";
				for(int i = 0; i < artist_parts.length(); ++i) {
					QString artist_current = artist_parts.at(i).toLower();
					QString artist_first = artist_current.at(0).toUpper();
					artist_current = artist_current.remove(0,1);
					artist_edited += artist_first + artist_current + " ";
				}
				artist = artist_edited.trimmed();

				ui->Lyric_Artist->setText(artist);
				ui->Lyric_Title->setText(title);

				showMessage(1,"Now Playing:", artist + " by " + title);

				if(ui->Lyric_AutoFetch->isChecked()) {
					RequestLyrics();
				}

			}

			this->setWindowTitle(title + " (" + artist + ") -- " + version);

		}

	// These contains the servername and the bitrate (several information is sent at once because of "msglevel all=4" mplayer preference)
	} else if(output.contains("Name   : ") && code_sel == played) {

		QString bitrate = QString(output.split("Bitrate: ").at(1).split("Cache size set to").at(0)).remove("\n");
		if(ui->Details_Bitrate->text() == "" || ui->Details_Bitrate->text() == "---") {
			ui->Details_Bitrate->setText(bitrate);
			played_values["bitrate"] = bitrate;
			QSqlQuery query;
			query.exec("UPDATE details SET bitrate='" + bitrate + "' WHERE code LIKE '" + played + "'");
			query.clear();
		}

		// NEEDS SOME ADDITIONS

	} else if(output.startsWith("MPlayer") && code_sel == played && output.contains("Playing")) {

		// NEEDS STILL TO BE DONE

	// Here the audio format is sent
	} else if(output.contains("Selected audio codec:") && code_sel == played) {

		QString codec;

		QStringList temp = output.split("Selected audio codec:");
		if(temp.size() > 1) {
			codec = temp.at(1);
		}

		temp = codec.split("[");
		if(temp.size() > 1) {
			codec = temp.at(1);
		}

		codec = codec.split("] ").at(0);

		if(played_values["bitrate"] != "") {
			ui->Details_Bitrate->setText(played_values["bitrate"] + " (" + codec + ")");
			played_values["codec"] += codec;
		}

		if(!ui->Details_Bitrate->text().endsWith(")")) {
			ui->Details_Bitrate->setText(ui->Details_Bitrate->text() + " (" + codec + ")");
			played_values["codec"] = codec;
		}

	}


	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}

}


// Get the streamripper for intelligent playback output
void MainWindow::GetStreamripperForPlayingOutput() {
	
	int order_column = ui->list->horizontalHeader()->sortIndicatorSection();
	int order_ascdesc = ui->list->horizontalHeader()->sortIndicatorOrder();

	ui->list->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder);

	QSqlQuery query;

	QString code = played;

	int row = -1;
	for(int t = 0; t < ui->list->rowCount(); ++t) {
		if(ui->list->item(t,6)->text() == code) {
			row = t;
		}
	}


	QString output = streamripperforplaying->readLine(0).trimmed();

	// SOMETHING DOESN'T WORK HERE YET!!!

//	if(output.startsWith("shutting down") || output.startsWith("bye")) {
//		QMessageBox::critical(this,"Error","An error occured! MPlayer will be stopped.\n\n\nPlease make first sure you have a working connection to the internet and that the URL is right.\n\nIf this is the case, then please report this as a bug to 'rripper91@yahoo.com'.\n(In the report, mention your version of RRipper and the URL you tried to connect with)");
//		values[code]["stoprecording"] = "1";
//	} else {
	if(1==1){

		// The procedure is basically the same as in the function 'GetStreamripperOutput()'

		if(output.trimmed() != "" && row != -1) {

			if(output.startsWith("Connecting")) {

				ui->list->item(row,1)->setText(status_connecting);

			}

			if(output.startsWith("stream")) {

				ui->list->item(row,1)->setText(status_receivingdetails);

				QString ReceivedName = output.split(": ").at(1);

				if(ReceivedName.trimmed() != ui->list->item(row,2)->text().trimmed()) {

					int msg = QMessageBox::Cancel;

					if(ui->Global_Matching->currentIndex() == 0) {

						msg = QMessageBox::Cancel;

					} else if(ui->Global_Matching->currentIndex() == 1) {

						msg = QMessageBox::Ok;

					} else if(ui->Global_Matching->currentIndex() == 2) {

						msg = QMessageBox::information(this,
								tr("Received Streamname"),
								tr("The received streamname differs from the saved one.\n\nSaved: %1\n Received: %2\n\nDo you want to adapt the new name?").arg(ui->list->item(row,2)->text()).arg(ReceivedName),
								QMessageBox::Ok | QMessageBox::Default, QMessageBox::Cancel);

					}

					if(msg == QMessageBox::Ok) {

						ui->list->item(row,2)->setText(ReceivedName);
						ui->Details_StreamNameEdit->setText(ReceivedName);
						ui->Details_StreamName->setText("<p align=\"center\" style=\"font-size: 12pt\"><b>" + ReceivedName + "</b></p>");

						query.exec("UPDATE details SET name='" + ReceivedName + "' WHERE code LIKE '" + code + "'");

						SQLERR(stations,query,"GetStreamripperForPlayingOutput 2");

						query.clear();

					}

				}

			}

			if(output.startsWith("server name")) {

				ui->list->item(row,1)->setText(status_receivingdetails);

				QString servername = output.split(": ").at(1);

				if(servername != ui->Details_ServerName->text()) {
					ui->Details_ServerName->setText(servername);
					query.exec("UPDATE details SET server='" + servername + "' WHERE code LIKE '" + code + "'");

					SQLERR(stations,query,"GetStreamripperForPlayingOutput 3");

					query.clear();
				}

			}

			if(output.startsWith("declared bitrate")) {

				ui->list->item(row,1)->setText(status_receivingdetails);

				QString bitrate = output.split(": ").at(1) + " kbit/s";

				if(bitrate != ui->Details_Bitrate->text()) {
					ui->Details_Bitrate->setText(bitrate);
					query.exec("UPDATE details SET bitrate='" + bitrate + "' WHERE code LIKE '" + code + "'");

					SQLERR(stations,query,"GetStreamripperForPlayingOutput 4");

					query.clear();
				}

			}

			if(output.startsWith("meta interval")) {

				ui->list->item(row,1)->setText(status_receivingdetails);

				QString metainterval = output.split(": ").at(1);

				if(metainterval != ui->Details_MetaInterval->text()) {
					ui->Details_MetaInterval->setText(metainterval);
					query.exec("UPDATE details SET metainterval='" + metainterval + "' WHERE code LIKE '" + code + "'");

					SQLERR(stations,query,"GetStreamripperForPlayingOutput 5");

					query.clear();
				}

			}

			if(output.startsWith("relay port")) {

				ui->list->item(row,1)->setText(status_receivingdetails);

				QString localhost = output.split(": ").at(1);

				playinglocalhost = localhost;

			}

			if(output.startsWith("[getting")) {

				ui->list->item(row,1)->setText(status_gettingtrackname);

			}

			if(output.startsWith("[buffering")) {

				ui->list->item(row,1)->setText(status_buffering);

			}

			if(output.startsWith("[skipping")) {

				ui->list->item(row,1)->setText(status_playing);

			}

			if(output.startsWith("[ripping")) {

				ui->list->item(row,1)->setText(status_playing);

				save_song->setEnabled(true);

			}

			if(output.startsWith("[buffering") || output.startsWith("[skipping") || output.startsWith("[ripping")) {

				// If localhost is available -> start playing
				if(playingstarted == 0 && playinglocalhost != "") {

					QStringList arguments;
					arguments << "http://localhost:" + playinglocalhost << "-cache" << "32" << "-msglevel" << "all=4"  << "-idle" << "-slave";
					mplayer->start(ui->Global_MPlayerPath->text(),arguments);
					playingstarted = 1;

				}

				output = output.remove("[skipping...   ]").trimmed();
				output = output.remove("[buffering...   ]").trimmed();
				output = output.remove("[ripping...    ]").trimmed();

				QString id3 = output.split("[").at(0).trimmed();
				QString Size = output.split("[").at(1).trimmed().split("]").at(0).trimmed();

				QString title;
				QString artist;

				QString artist_old = ui->list->item(row,3)->text().trimmed();
				QString title_old = ui->list->item(row,4)->text().trimmed();

				if(id3 == "") {

					title = tr("[No ID3 available]");
					artist = tr("[No ID3 available]");

				} else {

					QString split_string = "";

					int split = 0;
					int pos = 0;

					QRegExpValidator *id3_name = new QRegExpValidator(QRegExp("^([0-9]){0,3} (\\S|\\s)*$"),this);

					if(id3.contains("- -")) {

						split_string = "- -";
						split = 1;

					} else if(id3.contains(" - ")) {

						split_string = " - ";
						split = 1;

					} else if(id3.contains("--")) {

						split_string = "--";
						split = 1;

					} else if(id3_name->validate(id3,pos)) {

						QStringList split = id3.split(" ");
						split.removeFirst();
						id3 = split.join(" ");

					} else if(id3.contains("-")) {

						split_string = "-";
						split = 1;

					}

					if(split == 1) {

						artist = id3.split(split_string).at(0);
						title = id3.split(split_string).at(1);

					} else {

						artist = "";
						title = id3;

					}

					artist = artist.replace("_"," ");
					title = title.replace("_"," ");

					artist = artist.trimmed();
					title = title.trimmed();

					query.exec("SELECT swapartisttitle,location FROM details WHERE code LIKE '" + code + "'");
					query.next();

					QString swapartisttitle = query.value(query.record().indexOf("swapartisttitle")).toString();
					QString location = query.value(query.record().indexOf("location")).toString();

					SQLERR(stations,query,"GetStreamripperForPlayingOutput 6");

					query.clear();

					if(swapartisttitle == "1") {
						QString temp = artist;
						artist = title;
						title = temp;
					}

					// If the id3 has changed
					if(title != title_old && artist != artist_old && title != artist_old && artist != title_old) {

						if(swapartisttitle == "1") {
							QString temp = artist_old;
							artist_old = title_old;
							title_old = temp;
						}

						// Get the created file...
						QDir dir = QDir(QDir::tempPath() + "/rripper/");
						QStringList files;
						files = dir.entryList(QStringList("*"),QDir::Files | QDir::NoSymLinks, QDir::Time | QDir::Reversed);

						if(files.size() > 0) {
							QFile file(QDir::tempPath() + "/rripper/" + files.at(0));

							// ...and if 'Save' is checked -> save it; otherwise: delete it
							if(save_song->isChecked()) {

								QDir dir_location;
								if(!dir_location.exists(location)) {
									dir_location.mkpath(location);
								}

								if(file.fileName().endsWith(".mp3")) {
									if(!file.copy(location + artist_old + " - " + title_old + ".mp3")) {
										QMessageBox::critical(this,tr("Error"),tr("[MP3] Error!\nCopying of file failed!"));
									}
								} else if(file.fileName().endsWith(".ogg")) {
									if(!file.copy(location + artist_old + " - " + title_old + ".ogg")) {
										QMessageBox::critical(this,tr("Error"),tr("[OGG] Error!\nCopying of file failed!"));
									}
								}

							}

							file.remove();

							save_song->setChecked(false);

							playingFileID3 = id3;

						}


						state(1,"PLAYING: " + artist.toUpper() + " - " + title.toUpper());


						title = title.toLower();
						QStringList title_parts = title.split(" ");
						QString title_edited = "";
						if(title.size() > 0) {
							for(int i = 0; i < title_parts.length(); ++i) {
								QString title_current = title_parts.at(i).toLower();
								QString title_first = title_current.at(0).toUpper();
								title_current = title_current.remove(0,1);
								title_edited += title_first + title_current + " ";
							}
							title = title_edited.trimmed();
						}


						artist = artist.toLower();
						QStringList artist_parts = artist.split(" ");
						QString artist_edited = "";
						if(artist.size() > 0) {
							for(int i = 0; i < artist_parts.length(); ++i) {
								QString artist_current = artist_parts.at(i).toLower();
								QString artist_first = artist_current.at(0).toUpper();
								artist_current = artist_current.remove(0,1);
								artist_edited += artist_first + artist_current + " ";
							}
							artist = artist_edited.trimmed();
						}

						ui->list->item(row,3)->setText(artist);
						ui->list->item(row,4)->setText(title);

						ui->Lyric_Artist->setText(artist);
						ui->Lyric_Title->setText(title);

						showMessage(1,"Now Playing:", artist + " by " + title);


						if(ui->Lyric_AutoFetch->isChecked()) {
							RequestLyrics();
						}

					}

					this->setWindowTitle(title + " (" + artist + ") -- " + version);

					if(row == ui->list->selectedItems().at(0)->row()) {
						QString text;
						text = "<p align=\"center\">";
						if(artist != "") {
							text += "<b>" + artist + "</b> - ";
						}
						text += title + "</p>";
						ui->Details_ArtistTitle->setText(text);
					}

				}

				if(Size.endsWith("kb") || Size.endsWith("M")) {

					if(Size.endsWith("kb")) {
						Size = Size.remove(Size.size() - 2, Size.size()) + " KB";
					} else if(Size.endsWith("M")) {
						Size = Size.remove(Size.size() - 1, Size.size()) + " MB";
					}

					ui->list->item(row,5)->setText("(" + Size + ")");

				}

			}
		}
	}

	if(order_ascdesc == 0) {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::AscendingOrder);
	} else {
		ui->list->horizontalHeader()->setSortIndicator(order_column,Qt::DescendingOrder);
	}

	StartStopRecording();

}


// Another station directory is selected
void MainWindow::DirectoryChanged(QString directory) {

	QSqlQuery query(directories);

	ui->Directories_list->clear();
	expanded.clear();

	if(directory == "Shoutcast") {

		query.exec("SELECT COUNT(genre) FROM shoutcast_genres");
		query.next();

		QString amount = query.value(0).toString();

		query.clear();

		if(amount == "0") {
			RequestShoutcastGenres();
		} else {
			DisplayStations();
		}

	} else if(directory == "Icecast") {

		query.exec("SELECT COUNT(genre) FROM icecast_genres");
		query.next();

		QString amount = query.value(0).toString();

		query.clear();

		if(amount == "0") {
			RequestIcecastData();
		} else {
			DisplayStations();
		}

	}

	SQLERR(directories,query,"DirectoryChanged");

	query.clear();

}


// Request the Genres
void MainWindow::RequestShoutcastGenres() {

	state(1,"Requesting Shoutcast genre list");

	expanded.clear();

	QSqlQuery query(directories);
	query.exec("DELETE FROM shoutcast_stations");
	SQLERR(directories,query,"RequestShoutcastStations 1");
	query.clear();
	query.exec("DELETE FROM shoutcast_genres");
	SQLERR(directories,query,"RequestShoutcastStations 2");
	query.clear();

	ui->Directories_list->clear();

	ui->Directories_Directory->setEnabled(false);
	ui->Directories_FilterGenres->setEnabled(false);
	ui->Directories_list->setEnabled(false);
	ui->Directories_Download->setEnabled(false);
	ui->Directories_FilterStations->setEnabled(false);
	ui->Directories_FilterGenres_label->setEnabled(false);
	ui->Directories_FilterStations_label->setEnabled(false);
	ui->Directories_FilterBitrate->setEnabled(false);
	ui->Directories_FilterBitrate_label->setEnabled(false);
	ui->Directories_FilterBitrateDisplay->setEnabled(false);
	ui->Directories_allCollapse->setEnabled(false);

	// get genre list
	requestshoutcastgenres->get(QNetworkRequest(QUrl("http://yp.shoutcast.com/sbin/newxml.phtml")));

}


// Request the Icecast data
void MainWindow::RequestIcecastData() {

	state(1,"Requesting Icecast station list");

	expanded.clear();

	QSqlQuery query(directories);
	query.exec("DELETE FROM icecast_stations");
	SQLERR(directories,query,"RequestIcecastData 1");
	query.clear();
	query.exec("DELETE FROM icecast_genres");
	SQLERR(directories,query,"RequestIcecastData 2");
	query.clear();

	ui->Directories_list->clear();

	ui->Directories_Directory->setEnabled(false);
	ui->Directories_FilterGenres->setEnabled(false);
	ui->Directories_list->setEnabled(false);
	ui->Directories_Download->setEnabled(false);
	ui->Directories_FilterStations->setEnabled(false);
	ui->Directories_FilterGenres_label->setEnabled(false);
	ui->Directories_FilterStations_label->setEnabled(false);
	ui->Directories_FilterBitrate->setEnabled(false);
	ui->Directories_FilterBitrate_label->setEnabled(false);
	ui->Directories_FilterBitrateDisplay->setEnabled(false);
	ui->Directories_allCollapse->setEnabled(false);

	// get genre list
	requesticecastdata->get(QNetworkRequest(QUrl("http://dir.xiph.org/yp.xml")));

}


// An directory item is expanded (THIS FUNCTION COULD GET SOME IMPROVEMENT!!)
void MainWindow::DirectoriesItemExpanded(QTreeWidgetItem *item) {

	itemJustExpanded = item;

	if(ui->Directories_Directory->currentText() == "Shoutcast") {

		QString genre_text = item->text(0);

		QSqlQuery query(directories);

		query.exec("SELECT COUNT(name) FROM shoutcast_stations WHERE genre_searched LIKE '" + genre_text + "'");
		query.next();

		if(expanded.indexOf(item->text(0)) == -1) {
			expanded << item->text(0);
		}

		if(query.value(0).toString() == "0") {

//			ui->Directories_Directory->setEnabled(false);
			ui->Directories_list->setEnabled(false);
			ui->Directories_FilterGenres->setEnabled(false);
			ui->Directories_Download->setEnabled(false);
			ui->Directories_FilterStations->setEnabled(false);
			ui->Directories_FilterGenres_label->setEnabled(false);
			ui->Directories_FilterStations_label->setEnabled(false);
			ui->Directories_FilterBitrate->setEnabled(false);
			ui->Directories_FilterBitrate_label->setEnabled(false);
			ui->Directories_FilterBitrateDisplay->setEnabled(false);
			ui->Directories_allCollapse->setEnabled(false);

			// Send station list request
			requestshoutcaststations->get(QNetworkRequest(QUrl("http://yp.shoutcast.com/sbin/newxml.phtml?genre=" + item->text(0))));

			// This is the genre, the station list is loading
			genrecurrentlyloading = item->text(0);

		} else if(query.value(0).toString() != "0" && IComeFromDisplayStations == 0) {
			DisplayStations();
		} else {
			IComeFromDisplayStations = 0;
		}

		SQLERR(directories,query,"DirectoriesItemExpanded");

		query.clear();

	} else if(ui->Directories_Directory->currentText() == "Icecast") {

		if(expanded.indexOf(item->text(0)) == -1) {
			expanded << item->text(0);
		}

		if(IComeFromDisplayStations == 0 && item->childCount() < 2) {

			DisplayStations();

		} else {

			IComeFromDisplayStations = 0;

		}

	}

}


// An item is collapsed
void MainWindow::DirectoriesItemCollapsed(QTreeWidgetItem *item) {
	
	if(CurrentCollapsingOutOfFunction == 0) {

		expanded.removeOne(item->text(0));

	}

}


// Analyse the received data containing the shoutcast genres
void MainWindow::AnalyseShoutcastGenres(QNetworkReply *reply) {

	directories.transaction();

	QString genres = reply->readAll();

	QSqlQuery query(directories);
	query.prepare("INSERT INTO shoutcast_genres (genre) VALUES (:genre)");

	// Analyse the xml file
	QDomDocument doc("genrelist");
	doc.setContent(genres);
	QDomElement root = doc.documentElement();

	if(root.tagName() != "genrelist") {
		QMessageBox::critical(this,tr("Error"),tr("An error occured:\n\nThe received data doesn't fit the expected pattern."));
	} else {

		state(1,"Shoutcast genre list received");

		QDomNode n = root.firstChild();

		while(!n.isNull()) {

			QDomElement e = n.toElement();

			if(e.tagName() != "genre") {
				QMessageBox::critical(this,tr("Error"),tr("An error occured:\n\nUnexpected data received!"));
			} else {

				query.bindValue(":genre",e.attribute("name",""));

				query.exec();

			}

			n = n.nextSibling();

		}

	}

	directories.commit();

	SQLERR(directories,query,"AnalyseShoutcastGenres");

	query.clear();


	DisplayStations();

}


// Analysing the received data containing the received shoutcast stations
void MainWindow::AnalyseShoutcastStations(QNetworkReply *reply) {

	QString stations = reply->readAll();

	directories.transaction();

	QSqlQuery query(directories);
	query.prepare("INSERT INTO shoutcast_stations (name,description,mime,id,bitrate,genre_searched,genre_all) VALUES (:name,:description,:mime,:id,:bitrate,:genre_searched,:genre_all)");

	QDomDocument doc("stationlist");
	doc.setContent(stations);
	QDomElement root = doc.documentElement();

	if(root.tagName() != "stationlist") {
		QMessageBox::critical(this,ERR,ERRT + tr("\n\nAnalyseShoutcastStations: Received data doesn't fit expected pattern!"));
	} else {

		state(1,"Shoutcast station list received");

		QDomNode n = root.firstChild();

		while(!n.isNull()) {
			QDomElement e = n.toElement();

			if(e.tagName() == "station") {
				
				query.bindValue(":name",e.attribute("name","").split(" - ").at(0));

				QString description = "";
				QStringList temp = e.attribute("name","").split(" - ");
				if(temp.size() == 2) {
					description = temp.at(1);
				} else if(temp.size() > 2) {
					temp.removeFirst();
					description = temp.join(" - ");
				}
				if(description == "[SHOUTcast.com]") {
					description = "";
				} else if(description.contains("[SHOUTcast.com]")) {
					description.remove("[SHOUTcast.com]");
					description = description.trimmed();
				}
				query.bindValue(":description",description);

				query.bindValue(":mime",e.attribute("mt",""));
				query.bindValue(":id",e.attribute("id",""));
				query.bindValue(":bitrate",e.attribute("br",""));
				query.bindValue(":genre_searched",genrecurrentlyloading);
				query.bindValue(":genre_all",e.attribute("genre",""));
				query.exec();


			}

			n = n.nextSibling();
		}

	}

	genrecurrentlyloading = "";

	directories.commit();

	SQLERR(directories,query,"AnalyseShoutcastStations");

	query.clear();

	DisplayStations();

}


// Analyse the received data containing all the Icecast data
void MainWindow::AnalyseIcecastData(QNetworkReply *reply) {

	QString data = reply->readAll();

	directories.transaction();

	QSqlQuery query(directories);

	query.prepare("INSERT INTO icecast_genres (genre) VALUES (:genre)");

	// Analyse the xml file; First save the genres
	QDomDocument doc("genres");
	doc.setContent(data);
	QDomElement root = doc.documentElement();

	QStringList savedgenres;

	if(root.tagName() != "directory") {
		QMessageBox::critical(this,tr("Error"),tr("An error occured:\n\nThe received data doesn't fit the expected pattern."));
	} else {

		state(1,"Icecast station list received");

		QDomNode n = root.firstChild();

		while(!n.isNull()) {

			QDomElement e = n.toElement();

			if(e.tagName() == "entry") {

				QDomNode n_2 = e.firstChild();

				while(!n_2.isNull()) {

					QDomElement e_2 = n_2.toElement();

					if(e_2.tagName() == "genre") {

						QStringList list = e_2.text().split(" ");

						for(int i = 0; i < list.size(); ++i) {

							if(!savedgenres.contains(list.at(i))) {

								query.bindValue(":genre",list.at(i));

								query.exec();

								savedgenres << list.at(i);

							}

						}

					}

					n_2 = n_2.nextSibling();

				}
			}

			n = n.nextSibling();

		}

	}

	directories.commit();

	SQLERR(directories,query,"AnalyseIcecastGenres");

	query.clear();



	directories.transaction();

	query.prepare("INSERT INTO icecast_stations (name,url,mime,bitrate,genre) VALUES (:name,:url,:mime,:bitrate,:genre)");

	// Analyse the xml file; Second save the stations
	QDomDocument doc_2("stations");
	doc_2.setContent(data);
	QDomElement root_2 = doc_2.documentElement();

	if(root_2.tagName() != "directory") {
		QMessageBox::critical(this,tr("Error"),tr("An error occured:\n\nThe received data doesn't fit the expected pattern."));
	} else {

		QDomNode n = root_2.firstChild();

		while(!n.isNull()) {

			QDomElement e = n.toElement();

			if(e.tagName() == "entry") {

				QDomNode n_2 = e.firstChild();

				while(!n_2.isNull()) {

					QDomElement e_2 = n_2.toElement();

					if(e_2.tagName() == "server_name") {

						query.bindValue(":name",e_2.text());

					} else if(e_2.tagName() == "listen_url") {

						query.bindValue(":url",e_2.text());

					} else if(e_2.tagName() == "server_type") {

						query.bindValue(":mime",e_2.text());

					} else if(e_2.tagName() == "bitrate") {

						query.bindValue(":bitrate",e_2.text());

					} else if(e_2.tagName() == "genre") {

						query.bindValue(":genre",e_2.text());

					}

					n_2 = n_2.nextSibling();

				}

				query.exec();

			}

			n = n.nextSibling();

		}

	}

	directories.commit();

	SQLERR(directories,query,"AnalyseIcecastStations");

	query.clear();


	DisplayStations();

}


// Display the stations
void MainWindow::DisplayStations() {

	state(1,"Updating directory view");

	QString name_selectedbefore = "";
	QString id_selectedbefore = "";

	QList<QTreeWidgetItem*> list = ui->Directories_list->selectedItems();
	if(list.size() > 0) {
		name_selectedbefore = list.at(0)->text(0);
		id_selectedbefore = list.at(0)->text(1);
	}

	QString directory = ui->Directories_Directory->currentText();


	ui->Directories_list->clear();

	// enable the connected elements again
	ui->Directories_Directory->setEnabled(true);
	ui->Directories_list->setEnabled(true);
	ui->Directories_FilterGenres->setEnabled(true);
	ui->Directories_Download->setEnabled(true);
	ui->Directories_FilterStations->setEnabled(true);
	ui->Directories_FilterGenres_label->setEnabled(true);
	ui->Directories_FilterStations_label->setEnabled(true);
	ui->Directories_FilterBitrate->setEnabled(true);
	ui->Directories_FilterBitrate_label->setEnabled(true);
	ui->Directories_FilterBitrateDisplay->setEnabled(true);
	ui->Directories_allCollapse->setEnabled(true);

	QSqlQuery query(directories);

	QStringList toexpand;

	if(directory == "Shoutcast") {

		query.exec("SELECT genre FROM shoutcast_genres ORDER BY genre");

		int k = 0;

		while(query.next()) {

			// Genre filter
			if(query.value(0).toString().contains(ui->Directories_FilterGenres->text(), Qt::CaseInsensitive)) {

				QTreeWidgetItem *item = new QTreeWidgetItem((QTreeWidget*)0, query.value(0).toStringList());
				ui->Directories_list->addTopLevelItem(item);

				if(expanded.indexOf(query.value(0).toString()) != -1) {
					QSqlQuery query2(directories);
					query2.exec("SELECT * FROM shoutcast_stations WHERE genre_searched LIKE '" + query.value(0).toString() + "' ORDER BY name");
					int atleastone = 0;
					while(query2.next()) {
						atleastone = 1;
						QString name = query2.value(query2.record().indexOf("name")).toString();
						// station name filter
						if(name.contains(ui->Directories_FilterStations->text(), Qt::CaseInsensitive)) {
							int bitrate = ui->Directories_FilterBitrate->value();

							QString saved_bitrate = query2.value(query2.record().indexOf("bitrate")).toString();
							bool ok;
							// bitrate filter
							if(saved_bitrate.toInt(&ok,10) >= bitrate) {
								QTreeWidgetItem *new_child = new QTreeWidgetItem((QTreeWidget*)0, QStringList(name));

								QString tooltip = "";
								QString description = query2.value(query2.record().indexOf("description")).toString();
								if(description != "") {
									tooltip += "<div style=\"margin-bottom: 5px\"><i>" + description + "</i></div>";
								}
								tooltip += "Bitrate: " + saved_bitrate;
								tooltip += "<br>MIME type: " + query2.value(query2.record().indexOf("mime")).toString();
								tooltip += "<br>Genre: " + query2.value(query2.record().indexOf("genre_all")).toString();
								new_child->setToolTip(0,tooltip);

								new_child->setText(1,query2.value(query2.record().indexOf("id")).toString());

								ui->Directories_list->topLevelItem(k)->addChild(new_child);

							}

						}
					}
					query2.clear();

					if(atleastone == 0) {
						QTreeWidgetItem *new_child = new QTreeWidgetItem((QTreeWidget*)0, QStringList("Loading..."));
						ui->Directories_list->topLevelItem(k)->addChild(new_child);
					}
				} else {
					QTreeWidgetItem *new_child = new QTreeWidgetItem((QTreeWidget*)0, QStringList("Loading..."));
					ui->Directories_list->topLevelItem(k)->addChild(new_child);
				}

				if(expanded.indexOf(query.value(0).toString()) != -1) {
					toexpand << QString("%1").arg(k);
				}

				++k;

			}

		}

		SQLERR(directories,query,"DisplayStations 1");

		query.clear();

	} else if(directory == "Icecast") {

		QSqlQuery query(directories);
		query.exec("SELECT genre FROM icecast_genres ORDER BY genre");

		int k = 0;



		while(query.next()) {

			// genre filter
			if(query.value(0).toString().contains(ui->Directories_FilterGenres->text(), Qt::CaseInsensitive)) {

				QTreeWidgetItem *item = new QTreeWidgetItem((QTreeWidget*)0, query.value(0).toStringList());
				ui->Directories_list->addTopLevelItem(item);

				if(expanded.indexOf(query.value(0).toString()) != -1) {
					QSqlQuery query2(directories);
					query2.exec("SELECT * FROM icecast_stations WHERE genre LIKE '%" + query.value(0).toString() + "%' ORDER BY name");
					int atleastone = 0;
					while(query2.next()) {
						atleastone = 1;
						QString name = query2.value(query2.record().indexOf("name")).toString();
						// station name filter
						if(name.contains(ui->Directories_FilterStations->text(), Qt::CaseInsensitive)) {
							int bitrate = ui->Directories_FilterBitrate->value();

							QString saved_bitrate = query2.value(query2.record().indexOf("bitrate")).toString();
							bool ok;
							// bitrate filter
							if(saved_bitrate.toInt(&ok,10) >= bitrate) {
								QTreeWidgetItem *new_child = new QTreeWidgetItem((QTreeWidget*)0, QStringList(name));

								QString tooltip = "";

								tooltip += "Bitrate: " + saved_bitrate;
								tooltip += "<br>MIME type: " + query2.value(query2.record().indexOf("mime")).toString();
								tooltip += "<br>Genre: " + query2.value(query2.record().indexOf("genre")).toString();
								new_child->setToolTip(0,tooltip);

								new_child->setText(1,query2.value(query2.record().indexOf("url")).toString());

								ui->Directories_list->topLevelItem(k)->addChild(new_child);

							}

						}
					}
					query2.clear();

					if(atleastone == 0) {
						QTreeWidgetItem *new_child = new QTreeWidgetItem((QTreeWidget*)0, QStringList("Loading..."));
						ui->Directories_list->topLevelItem(k)->addChild(new_child);
					}
				} else {
					QTreeWidgetItem *new_child = new QTreeWidgetItem((QTreeWidget*)0, QStringList("Loading..."));
					ui->Directories_list->topLevelItem(k)->addChild(new_child);
				}

				if(expanded.indexOf(query.value(0).toString()) != -1) {
					toexpand << QString("%1").arg(k);
				}

				++k;

			}

		}

		SQLERR(directories,query,"DisplayStations 1");

		query.clear();
		


	}


	for(int i = 0; i < toexpand.size(); ++i) {

		IComeFromDisplayStations = 1;
		bool ok;
		QModelIndex index = ui->Directories_list->model()->index(toexpand.at(i).toInt(&ok,10),0,QModelIndex());
		ui->Directories_list->setExpanded(index,true);

	}


	QList<QTreeWidgetItem*> temp = ui->Directories_list->findItems(name_selectedbefore,Qt::MatchExactly,0);
	if(temp.size() == 1) {
		temp.at(0)->setSelected(true);
	} else if(temp.size() > 1) {

		for(int i = 0; i < temp.size(); ++i) {
			if(temp.at(i)->text(1) == id_selectedbefore) {
				temp.at(i)->setSelected(true);
			}
		}

	}

	if(expanded.size() > 0) {
		ui->Directories_list->scrollToItem(itemJustExpanded,QAbstractItemView::EnsureVisible);
	}

}


// Add a station to the list (double click)
void MainWindow::AddStationToList(QTreeWidgetItem *item) {

	AddNewStation();

	int index = -1;

	for(int i = 0; i < ui->list->rowCount(); ++i) {
		if(ui->list->item(i,6)->text() == codeOfLastAdded) {
			index = i;
		}
	}

	if(index == -1) {

		QMessageBox::critical(this,tr("Error"),tr("Error while adding station to list."));

	} else {

		QSqlQuery query(directories);

		if(ui->Directories_Directory->currentText() == "Shoutcast") {

			query.exec("SELECT name,id FROM shoutcast_stations WHERE id LIKE '" + item->text(1) + "'");
			query.next();

			ui->Details_StreamNameEdit->setText(query.value(query.record().indexOf("name")).toString());
			ui->Details_URLEdit->setText("http://yp.shoutcast.com/sbin/tunein-station.pls?id=" + query.value(query.record().indexOf("id")).toString());

		} else if(ui->Directories_Directory->currentText() == "Icecast") {

			query.exec("SELECT name,url FROM icecast_stations WHERE url LIKE '" + item->text(1) + "'");
			query.next();

			ui->Details_StreamNameEdit->setText(query.value(query.record().indexOf("name")).toString());
			ui->Details_URLEdit->setText(query.value(query.record().indexOf("url")).toString());

		}

		SQLERR(directories,query,"AddStationToList");

		query.clear();

		UpdateStationsDatabase();

	}

	state(1,"Added station '" + item->text(0) + "' to list");

}


// Collapse all the items
void MainWindow::DirectoryAllCollapse() {

	CurrentCollapsingOutOfFunction = 1;

	for(int i = 0; i < expanded.size(); ++i) {

		QTreeWidgetItem *item = ui->Directories_list->findItems(expanded.at(i),Qt::MatchExactly,0).at(0);

		item->setExpanded(false);

	}

	expanded.clear();

	CurrentCollapsingOutOfFunction = 0;

}


void MainWindow::FirstLyricsSourceChanged() {

	if(ui->Global_FirstLyricsSource->currentIndex() == 0) {
		lyricsSource = 0;
	} else {
		lyricsSource = 2;
	}

}


// Request the lyrics
void MainWindow::RequestLyrics() {

	ui->Lyric_Artist->setEnabled(false);
	ui->Lyric_Artist_label->setEnabled(false);
	ui->Lyric_AutoFetch->setEnabled(false);
	ui->Lyric_Display->setEnabled(false);
	ui->Lyric_Fetch->setEnabled(false);
	ui->Lyric_Reset->setEnabled(false);
	ui->Lyric_Title->setEnabled(false);
	ui->Lyric_Title_label->setEnabled(false);
	ui->Global_FirstLyricsSource->setEnabled(false);

	// request only when stream is played (otherwise senseless)
	if(played != "") {

		QString artist = ui->Lyric_Artist->text().trimmed();
		QString title = ui->Lyric_Title->text().trimmed();

		if(artist.contains("(")) {
			artist = artist.split("(").at(0);
		}

		if(title.contains("(")) {
			title = title.split("(").at(0);
		}

		artist = artist.replace(" & "," and ");
		artist = artist.replace("&"," and ");
		artist = artist.replace(" ","-");

		title = title.replace(" & "," and ");
		title = title.replace("&"," and ");
		title = title.replace(" ","-");

		title = title.replace("-And-","-and-");
		title = title.replace("-A-","-a-");
		title = title.replace("-As-","-as-");
		title = title.replace("-On-","-on-");

		title = title.replace("--","-");

		state(1,"Requesting Lyrics from Lyricsvip.com. Artist: " + artist.toUpper() + " Title: " + title.toUpper());
		requestlyrics->get(QNetworkRequest(QUrl("http://www.lyricsvip.com/" + artist + "/" + title + "-Lyrics.html")));

	}

}


// Analyse and display the lyrics
void MainWindow::AnalyseLyrics(QNetworkReply *reply) {

	QString text = reply->readAll().trimmed();

		QStringList text_split = text.split("<table class=\"tbl0\"><tr><td class=\"td1\">");
		if(text_split.size() > 1) {

			text = text_split.at(1);


			text = text.split("</td></tr></table>").at(0);

			text = text.replace("<br />","<br>");
			text = text.replace(" Lyrics</h1>","</h1>");

			text.replace("h2>","h3>");
			text.replace("h1>","h2>");

			text.replace("</h3>","</h3><div style=\"font-size: 8pt\">");


			text = text + "</div><br><br>";

			UpdateLyricsInterfaces("LyricsVIP.com",text,1);

		} else {

			UpdateLyricsInterfaces("LyricsVIP.com","<i>[No Lyrics Found]</i>",1);

		}

}


void MainWindow::UpdateLyricsInterfaces(QString source,QString lyrics,int enabled) {

	ui->Lyric_Artist->setEnabled(enabled);
	ui->Lyric_Artist_label->setEnabled(enabled);
	ui->Lyric_AutoFetch->setEnabled(enabled);
	ui->Lyric_Display->setEnabled(enabled);
	ui->Lyric_Fetch->setEnabled(enabled);
	ui->Lyric_Reset->setEnabled(enabled);
	ui->Lyric_Title->setEnabled(enabled);
	ui->Lyric_Title_label->setEnabled(enabled);

	ui->Lyric_Source->setText(source);

	ui->Lyric_Display->setText(lyrics);
	external->displayLyrics(lyrics);
	ui->Lyric_ScrollArea->ensureVisible(0,0);

}


// Reset the lyrics fields to the current played station id3
void MainWindow::ResetLyricFields() {

	int row = ui->list->findItems(played,Qt::MatchExactly).at(0)->row();

	ui->Lyric_Artist->setText(ui->list->item(row,3)->text());
	ui->Lyric_Title->setText(ui->list->item(row,4)->text());

	RequestLyrics();

}


// Set/Unset auto-Fetch
void MainWindow::LyricAutoFetch() {

	SaveGlobalPreferences();

	if(ui->Lyric_AutoFetch->isChecked() && played != "") {
		RequestLyrics();
	}

}


// Kill all internal processes
void MainWindow::KillAllInternalProcesses() {

	int msg = QMessageBox::warning(this,tr("Stop All Internal Processes?"),tr("Do you really want to stop all internally started MPlayer & Streamripper processes? This will stop all recordings/playing.\n\nThis can be a solution if a internally started process of MPlayer/Streamripper wasn't stopped correctly."), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

	if(msg == QMessageBox::Yes) {

		for(int i = 0; i < ui->list->rowCount(); ++i) {

			QString code = ui->list->item(i,6)->text();

			if(values[code]["recording"] == "1") {

				values[code]["stoprecording"] = "1";

			} else {

				if(stations_processes[code]) {
					if(stations_processes[code]->state() == QProcess::Running) {
						stations_processes[code]->kill();
					}
				}

				stations_processes[code] = new QProcess;

			}

		}

		state(1,"Killed all streamripper processes");

		if(values[played]["playing"] == "1") {
			if(ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text() != played) {
				StartStopPlaying();
			}

			StartStopPlaying();

		} else {
			played = "";
		}

		StartStopRecording();

		mplayer->terminate();
		while(mplayer->waitForFinished(2000)) {
		}

		state(1,"MPlayer process terminated");

		if(streamripperforplaying->state() == QProcess::Running) {
			streamripperforplaying->kill();
		}

	}

}


// Kill all external processes
void MainWindow::KillAllExternalProcesses() {

	int msg = QMessageBox::warning(this,tr("WARNING!!"),tr("WARNING!!\n\nThis forces all *EXTERNAL* and internal Streamripper/MPlayer processes to stop!\n\nThis can affect other programs using MPlayer/Streamripper!\n\nDo you really want to continue??"),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);

	if(msg == QMessageBox::Yes) {

		QStringList arguments;
		arguments << "streamripper" << "mplayer";

		QProcess *killall = new QProcess;

		killall->start("/usr/bin/killall",arguments);

		while(killall->waitForFinished(2000)) {
		}

		state(1,"All EXTERNAL streamripper/mplayer processes killed");


		for(int i = 0; i < ui->list->rowCount(); ++i) {

			QString code = ui->list->item(i,6)->text();

			if(values[code]["recording"] == "1") {
				values[code]["stoprecording"] = "1";
				StartStopRecording();
			} else {

				if(stations_processes[code]) {
					if(stations_processes[code]->state() == QProcess::Running) {
						stations_processes[code]->kill();
					}
				}

			}

		}

		state(1,"Killed all streamripper processes");

		if(values[played]["playing"] == "1") {
			if(ui->list->item(ui->list->selectedItems().at(0)->row(),6)->text() != played) {
				StartStopPlaying();
			}

			StartStopPlaying();
		} else {
			played = "";
		}

		mplayer->terminate();
		while(mplayer->waitForFinished(2000)) {
		}

		state(1,"MPlayer process terminated");

		if(streamripperforplaying->state() == QProcess::Running) {
			streamripperforplaying->kill();
		}

	}

}


// Show a short note
void MainWindow::AboutRRipper() {

	QMessageBox msg;

	msg.setText("<p align=\"center\"><h1>" + version + "</h1></p>");
	msg.setIconPixmap(QPixmap(":/images/Logo_100.png"));
	msg.setInformativeText(tr("<p align=\"center\">Programmed by: Lukas Spies, 2010<br>Licenced under the GPL<br><br>Project Website: http://sourceforge.net/projects/rripper<br><br>E-Mail: rripper91@yahoo.com</p>"));
	QPushButton *ok = msg.addButton(tr("Close"), QMessageBox::ActionRole);
	msg.setDefaultButton(ok);

	msg.exec();

	state(1,"About RRipper");

}


// Closing RRipper (warning if active)
void MainWindow::closeEvent(QCloseEvent *event) {

	if(quitOrCloseRRipper == 0 && ui->Global_TrayIcon->isChecked()) {
		external->hide();
		this->hide();
		event->ignore();
		state(1,"RRipper minimized to Tray");
	} else {
		QApplication::setQuitOnLastWindowClosed(true);
		event->accept();
	}

}


void MainWindow::QuitProgram() {

	quitOrCloseRRipper = 0;

	int active = 0;

	if(played != "") {
		active = 1;
	} else {
		for(int i = 0; i < ui->list->rowCount(); ++i) {
			if(values[ui->list->item(i,6)->text()]["recording"] == "1") {
				active = 1;
			}
		}
	}

	int quit = 0;

	if(active == 1) {
		int msg = QMessageBox::warning(this,tr("Warning!"),tr("RRipper is still active. If you close RRipper, all Recordings/Playings will be stopped.\n\nContinue?"),QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);
		if(msg == QMessageBox::Yes) {
			quit = 1;
		}
	} else {
		quit = 1;
	}


	if(quit == 1) {

		state(1,"Quitting");

		QMap<QString, QProcess*>::const_iterator i = stations_processes.constBegin();

		while (i != stations_processes.constEnd()) {

			i.value()->kill();
			i.value()->disconnect();

			++i;

		}

		stations.close();

		if(stations.lastError().text().trimmed() != "") {
			QMessageBox::critical(this,ERR,ERRT + "\n\ncloseEvent: " + stations.lastError().text());
		}

		directories.close();

		if(directories.lastError().text().trimmed() != "") {
			QMessageBox::critical(this,ERR,ERRT + "\n\ncloseEvent: " + directories.lastError().text());
		}

		mplayer->terminate();
		while(mplayer->waitForFinished(2000)) {
		}

		streamripperforplaying->kill();

		QFile file(QDir::homePath() + "/.RRipper/.locked");
		QTextStream output(&file);
		int timestamp = QDateTime::currentDateTime().toTime_t();
		timestamp -= 100;
		output << timestamp;
		file.close();

		quitOrCloseRRipper = 1;
		this->close();

	} else {

		state(1,"Quitting cancelled");

		quitOrCloseRRipper = 0;

	}

}


// If the window is resized -> resize column width
void MainWindow::resizeEvent(QResizeEvent *event){

	ResizeColumns();

	event->accept();

}


MainWindow::~MainWindow() {
	delete ui;
}


void MainWindow::changeEvent(QEvent *e) {
	QMainWindow::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
    }
}

